// File:	SCommDll52.h
// Module:	SCommDll
// Author:	Jean-Pierre Eugster
//
// Copyright (C) Saia-Burgess Controls AG, CH-3280 Murten, 1993-2012
// @Doc HEADER INTERNAL EXTERNAL SCOMMDLL
//
// DESCRIPTION
// @module SCommDll52.h | Header file for SCommDll52.Dll low level interface DLL.<nl>
//  Last updated: $Id$
/*
@group1 REVISION HISTORY
@normal
10.10.2001<tab>Add new PCD types: PCD7: (L-Module) and PCS1: (Kompakt Regler)<nl>
11.09.2001<tab>Change PCDEXTHDR field <e PCDEXTHDR.IPNode> from BYTE to WORD<nl>
30.09.2002<tab>Changed PCD_MAXSTR from 1999 to 5999 and PCD_MAXSB from 31 to 95<nl>
17.10.2002<tab>Added PCD_FEATURE_EXTENDEDSFC flag value for <f PcdGetFwFeatures><nl>
10.04.2003<tab>Added PCD_FWDNLD_KEY in <t enumCodeFwDownload><nl>
26.09.2003<tab>Replaced <f PcdRdExtHeader> and <f PcdWrExtHeader> with <f PcdRdExtHeader2> and <f PcdWrExtHeader2><nl>
26.09.2003<tab>Replaced <t PCDEXTHDR> with PCDEXTHDR2, added <t PCDPROFISBUSDATA><nl>
23.10.2003<tab>Matt: Added 'BYTE IPType' to PCDEXTHDR2.<nl>
29.01.2004<tab>Added DWORD dwPcdFlags to PCDEXTHDR2.<nl>
20.04.2004<tab>Matt: Added PCDIPMODULEDATA structrure to PCDEXTHDR2.<nl>
10.08.2005<tab>Increase PCD_MAXXOB to 63 for PCD3.<nl>
24.10.2005<tab>Add new telegram <f PcdMutliMedia>.<nl>
31.10.2005<tab>Matt: Add "BYTE bIPDHCP; //Field use Dynamic Host Configuration Protocol".<nl>
25.04.2005<tab>Matt: Add PCDOPTFLAG_TERMINATION.<nl>
02.10.2006<tab>Add new PCD_SDATA mode to serial line S-Bus <t enumSBusModes> and PCD_MODESDATA in <t enumSBusModesToTry><nl>
27.11.2007<tab>Add new functions for read/write of temporary registers/flags<nl>
13.12.2007<tab>Change PCD_MAXFB from 999 to 1999, PCD_MAXPB from 299 to 999 and PCD_MAXCOB from 15 to 31<nl>
07.02.2008<tab>New functions <f PcdReadOpendata> and <f PcdWriteOpendata><nl>
24.06.2008<tab>Added PCD_FWDNLD_HEADER and PCD_FWDNLD_FLASHCRD in <t enumCodeFwDownload><nl>
09.07.2008<tab>Added <f PcdRdTaskData> and <f PcdWrTaskData> telegrams<nl>
31.07.2008<tab>Change PCD_MAXTEXTLEN and add PCD_MAXTEXTLEN1<nl>
11.05.2009<tab>new PCD_FWDNLD_CHKDNLDHEADER for FW downloader<nl>
07.12.2009<tab>Matt: Added PCD_MAXFLAGEX declaration.<nl>
18.12.2011<tab>Added <f PcdRdPeriData> and <f PcdWrPeriData>.<nl>
20.06.2013<tab>Matt: PCDIPMODULEDATA.bIPPguPort is not BOOL anymore, changed name to IPType.<nl>
*/

#ifdef SCOMMDLL_EXPORTS
#define SCOMMDLL_API __declspec(dllexport)
#else
#define SCOMMDLL_API __declspec(dllimport)
#endif

#ifdef SCOMMUSR_EXPORTS
#define SCOMMUSR_API __declspec(dllexport)
#else
#define SCOMMUSR_API __declspec(dllimport)
#endif

///////////////////////////////////////////////////////////////////////
// Types

#ifndef __LPCXXX__
#define __LPCXXX__
//@Type LPCBYTE | Constant long pointer to an unsigned char (BYTE).
typedef const unsigned char* LPCBYTE;
//@Type LPCWORD | Constant long pointer to an unsigned short (WORD).
typedef const unsigned short* LPCWORD;
//@Type LPCDWORD | Constant long pointer to an unsigned long (DWORD).
typedef const unsigned long* LPCDWORD;
#endif //__LPCXXX__

///////////////////////////////////////////////////////////////////////
// Exported variable

//extern SCOMMDLL_API int TraceFctLevel;

///////////////////////////////////////////////////////////////////////
// Macros

/*
@macro | _HIBYTE | High byte of a WORD (can be used as Lvalues).<nl>Replaces
 those used by windows which cannot be used as Lvalues
@parm WORD | w | WORD (16 bits unsigned integer)
@syntax _HIBYTE(w)
*/
#define _HIBYTE(w) (*((LPBYTE)&(w)+1))

/*
@macro | _LOBYTE | Low byte of a WORD (can be used as Lvalues).<nl>Replaces
 those used by windows which cannot be used as Lvalues
@parm WORD | w | WORD (16 bits unsigned integer)
@syntax _LOBYTE(w)
*/
#define _LOBYTE(w) (*((LPBYTE)&(w)))

/*
@macro | _HIWORD | High WORD of a DWORD (can be used as Lvalues).<nl>Replaces
 those used by windows which cannot be used as Lvalues
@parm DWORD | dw | DWORD (32 bits unsigned integer).
@syntax _HIWORD(dw)
 */
#define _HIWORD(dw) (*((LPWORD)&(dw)+1))

/*
@macro | _LOWORD | Low WORD of a DWORD (can be used as Lvalues).<nl>Replaces
 those used by windows which cannot be used as Lvalues
@parm DWORD | dw | DWORD (32 bits unsigned integer)
@syntax _LOWORD(dw)
*/
#define _LOWORD(dw) (*((LPWORD)&(dw)))

/*
@Enum ComStatus values, returned by comms functions.<nl>
 The function <f PcdMessage> will display the corresponding error information.
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumComStatus
{
	PCD_OK       =  0,	//@Emem Communications OK (0)
	PCD_NAK      =  1,	//@Emem NAK response, command not accepted (1)
	PCD_BCCERR   =  3,	//@Emem Invalid BCC or CRC, no other errors (3)
	PCD_INVALID  =  4,	//@Emem Invalid response (4)
	PCD_P8CONN   =  5,	//@Emem Bad connection between P800 and PCD6 (5)
	PCD_NORESP   =  6,	//@Emem No response (6)
	PCD_NORESP2  =  7,	//@Emem No response after PcdRestart in PcdWrAllocation (7)
	PCD_NODCD    =  8,	//@Emem No Data Carrier Detect (8)
	PCD_INUSE    =  9,	//@Emem Port in use in another mode or baud rate (9)
	PCD_BADPARM  = 10,	//@Emem Bad parameter (10)
	PCD_VERIFERR = 11,	//@Emem PCD memory write verify error (11)
	PCD_OUTOFMEM = 12,	//@Emem PC out of memory (12)
	PCD_WRONGCPU = 13,	//@Emem Physically connected to wrong CPU on PCD6 (13)
	PCD_NOCPU    = 14,	//@Emem CPU not present (14)
	PCD_TOFLASH  = 15,	//@Emem PCD to flash failed on M170 (15)
	PCD_DIFF     = 16,	//@Emem Files & CPU don't compare, see <f PcdCompare> (16)
	PCD_CANTOPEN = 17,	//@Emem Can't open file (17)
	PCD_WRERROR  = 18,	//@Emem Error writing to file (18)
	PCD_RDERROR  = 19,	//@Emem Error reading file (19)
	PCD_BADFILE  = 20,	//@Emem Invalid PCD file (20)
	PCD_CSERROR  = 21,	//@Emem PCD file checksum error (21)
	PCD_BADHDR   = 22,	//@Emem Invalid header in PCD (22)
	PCD_CODESIZE = 23,	//@Emem Not enough space in code segment (23)
	PCD_TEXTSIZE = 24,	//@Emem Not enough space in text segment (24)
	PCD_EXTSIZE  = 25,	//@Emem Extension memory segment too small (25)
	PCD_NOTINIT  = 26,	//@Emem Header not initialised (26)
	PCD_EPROM    = 27,	//@Emem Can't write EPROM memory (27)
	PCD_CANCEL   = 28,	//@Emem Operation cancelled by user (28)
	PCD_NOTDEF   = 29,	//@Emem Block not defined (29)
	PCD_RUNNING  = 30,	//@Emem CPU must be in Stop or Halt (30)
	PCD_NOTPROG  = 31,	//@Emem CPU not programmed (31)
	PCD_DRVOVER  = 32,  //@Emem No response Driver overlaoded.(32)<nl>
						// When this errro occurs, the client must reduce the activity until this condition dissapears
	PCD_SPECIAL  = 32,  //@Emem No response Driver overlaoded.(32)
	PCD_NOCHGEXT = 34,	//@Emem Can't change size of extension memory (34)
	PCD_NOTOPEN  = 35,	//@Emem COM port is not open or PCDCONN handle is invalid(35)
	PCD_TXERROR  = 36,	//@Emem Write to PCD failed (36)
	PCD_COMBUSY  = 37,	//@Emem Port request timeout (37)
	PCD_LOCKED   = 40,	//@Emem Port is in use (40)
	PCD_BUG      = 44,	//@Emem Fatal internal error (44)
	PCD_NOTSUP   = 45,	//@Emem Operation not supported by PCD F/W version (45)
	PCD_ALRDYDEF = 46,	//@Emem Station number already defined (46)
	PCD_EXTHDR   = 47,	//@Emem Not enough space for extended header (47)
	PCD_BADPWRD  = 48,	//@Emem Invalid password (48)
	PCD_EXTHDRCS = 49,	//@Emem Extended header checksum error (49)
	PCD_WRONGSTN = 50,	//@Emem Download to wrong S-Bus station ($STN) (50)
	PCD_WRONGFW  = 51,	//@Emem Download to PCD with wrong firmware ($PCDVER) (51)

	PCD_NOPARITY = 52,	//@Emem S-Bus Parity mode is not supported for this COM port (52)
	PCD_CHKIPADDS = 53, //@Emem Cannot download configuration, IP address would change (53)
	PCD_NORTSCTS = 54,	//@Emem RTS/CTS mode is not supported for this COM port (54)
	PCD_WRONGPCDTYPE = 55,  //@Emem Connected PCD's type does not match hardware settings (55)
	PCD_SQLERR   = 56,	//@Emem:(BUES) Error in Sql Statement (56)

    PCD_INTDATA  = 63,  //@Emem Internal data invalid (63)
    PCD_EXCEPTION= 64,  //@Emem Data buffer mismatch (64)
	PCD_ASKPWRD  = 65,	//@Emem Ask for password (65)
    PCD_NOTIMEOUT= 67,  //@Emem Timeout is not supported for this COM port (67)
    PCD_NODRIVER = 68,  //@Emem Driver not available (68)
	PCD_BUES_E   = 69,	//@Emem:(BUES) Received a BUES CMD_E response (69)
	PCD_BUES_A   = 70,	//@Emem:(BUES) Received a BUES CMD_A response (70)
	PCD_FDL_SAP  = 71,	//@Emem Cannot Activate SAP
	PCD_BLKXSUM  = 72,	//@Emem Invalid check sum in Block data (72)
	PCD_BLKABORT = 73,	//@Emem Block operation aborted by FW (73)
	PCD_BLKSEQ   = 74,	//@Emem Block operation out of sequence (74)
	PCD_OVRFLOW  = 75,  //@Emem Input parameter to big, will cause overflow (75)
	PCD_SYNTAX   = 76,  //@Emem Syntax error in input string (76)

// internal only (the user can't receive these messages)
// but user can receive the WM_COPYDATA / DRV_CALLBACK message to the CallBack window.

	DRV_RETRY	  =100,	//@Emem:(INTERNAL) retry (100)
	DRV_IOPENDING =101,	//@Emem:(INTERNAL) Input/Output pending (101)
	DRV_CALLBACK  =102,	//@Emem:(INTERNAL) Callback message from the driver (102)
	DRV_NOIMP     =103,	//@Emem:(INTERNAL) Not implemented function (103)
    DRV_PASSWORD  =104, //@Emem:(INTERNAL) Driver ask for password (104)
    DRV_DISCONNECT=105, //@Emem:(INTERNAL) Port disconnected (105)
    DRV_ENQUERY   =106, //@Emem:(INTERNAL) Acknowledge in P800 ACK-ENQ (106)
    DRV_SEQNUM    =107, //@Emem:(INTERNAL) Sequence number fault (107)
	DRV_REJECTED  =108, //@Emem:(INTERNAL) Function was rejected dureto expired timeout
};

//@Type COMSTATUS | Typedef enum <t enumComStatus> COMSTATUS;
typedef enum enumComStatus COMSTATUS;

/*
@Enum PCD Status<nl>
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
@Xref <f PcdRdStatus>
*/
enum enumPCDStatus
{
	PCD_OFFLINE = 0,	//@Emem PCD is offline (0)
	PCD_DISCONN = 1,	//@Emem PCD is disconnected (1)
	PCD_RUN     = 2,	//@Emem PCD is running (2)
	PCD_STOP    = 3,	//@Emem PCD is stopped (3)
	PCD_HALT    = 4,	//@Emem PCD is halted (4)
	PCD_CRUN    = 5,	//@Emem PCD is in conditional run (5)
	PCD_BADSTAT = 6,	//@Emem Unknown CPU status (6)
	PCD_CSTOP   = 7,	//@Emem These two are never returned, the PCD (7)
	PCD_TRACE   = 8,	//@Emem returns PCD_STOP instead for these modes (8)
    PCD_FWBOOT  = 9,    //@Emem PCD is in boot mode, no firmware is running (9)
	PCD_NOTRDY  = 10,	//@Emem PCD is not yet ready to accept telegrams (10)
	PCD_STANDBY = 11,	//@Emem PCD is in standby mode (11)
};

/*
@Enum Communication ports name<nl>
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
@Devnote The value of this table are very important an can only be changed if changes are
 reflected in the code. Each number maps to an index into a table.<nl>When adding new ports, check
 the code for any bad side effects<nl>PCD_FIRST_COM and PCD_LAST_COM should be adjusted when
 new serial line com ports are added, eg PCD_COM33.

@Xref <f PcdGetPortName>
*/
enum enumComPort
{
	PCD_COM    =  0,					//@Emem Serial line (COM1 to COM32) (0)
	PCD_TAPI   =  1,				    //@Emem TAPI Modem communications (1)
	PCD_RAS    =  2,					//@Emem RAS+TCP/IP communications (2)
	PCD_PC104  =  3,					//@Emem Dual-port RAM for PC/104 communications (3)
	PCD_BUES   =  4,					//@Emem BUES over serial line (4)
	PCD_SOCKET =  5,					//@Emem TCP/IP communications (5)
	PCD_SBUS_USB = 6,					//@Emem S-Bus USB communivation (6)
	PCD_PROFI_SBUS = 7,					//@Emem S-Bus Profi-S-Bus communication (7)

    PCD_FIRST_PORT = PCD_COM,		    //@Emem First port number (0)
    PCD_LAST_PORT  = PCD_PROFI_SBUS,	//@Emem Last port number (7)
};


/*
@Enum Baud rate table<nl>
@Comm The value are used in <t PCDSBUSDATA>.<nl>
 Don't use these value in the PCDOPENDATA structure. Use Baudrate value instead.
*/
enum enumBaudRate
{
	PCD_110    = 0,		//@Emem 110 bauds (0)
	PCD_150    = 1,		//@Emem 150 bauds (1)
	PCD_300    = 2,		//@Emem 300 bauds (2)
	PCD_600    = 3,		//@Emem 600 bauds (3)
	PCD_1200   = 4,		//@Emem 1200 bauds (4)
	PCD_2400   = 5,		//@Emem 2400 bauds (5)
	PCD_4800   = 6,		//@Emem 4800 bauds (6)
	PCD_9600   = 7,		//@Emem 9600 bauds (7)
	PCD_19200  = 8,		//@Emem 19200 bauds (8)
	PCD_38400  = 9,		//@Emem 38400 bauds (9)
    PCD_76800  = 10,    //@Emem 76800 baud (10)
    PCD_57600  = 11,	//@Emem 57600 bauds (11)
    PCD_115200 = 12,	//@Emem 115200 bauds (12)
    PCD_LASTBAUDRATE = 12   //@Emem last baud rate value
};

/*
@Enum Comms protocol, for PcdComOpen()<nl>
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumProtocol
{
	PCD_P800 = 0,		//@Emem PGU P800 (0)
	PCD_SBUS = 1,		//@Emem S-Bus (1)
};

#define PCD_FIRST_PROTOCOL PCD_P800
#define PCD_LAST_PROTOCOL PCD_SBUS

/*
@Enum S-Bus modes<nl>
@Comm When using the new PCD_SDATA mode, a 4 bytes header is added to each telegram, to tell
 the size and the sequence number of the telegram.<nl>PCD with older FW will simply ignore those
 extra 4 bytes, allowing to mix PCD with the old PCD_DATA mode and the new PCD_SDATA mode
 on the same port with RS485.<nl>
 This new mode should be used for modem communication to ensure extra reliability.<nl>
 Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumSBusModes
{
	PCD_PARITY = 0,		//@Emem Parity mode (0)
	PCD_BREAK  = 1,		//@Emem Break mode (1)
	PCD_DATA   = 2,		//@Emem Data mode (2)
	PCD_SDATA  = 3,		//@Emem Sequenced Data mode (3)
};

/*
@Enum S-Bus modes to try<nl>
 Those bit values can be combined by adding them together
@Comm When using PCD_MODEDATA and PCD_MODESDATA together, only the new sequenced data mode is tried.<nl>
 In order to use the old data mode, do not set the bit for PCD_MODESDATA.
 Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumSBusModesToTry
{
	PCD_MODEDATA    = 1,		//@Emem Data mode (1)
	PCD_MODEBREAK   = 2,		//@Emem Break mode (2)
	PCD_MODEPARITY  = 4,		//@Emem Parity mode (4)
	PCD_MODESDATA  =  8,		//@Emem Sequenced Data mode (8)
};

#define PCD_FIRST_MODE 0
#define PCD_LAST_MODE (PCD_MODEDATA|PCD_MODEBREAK|PCD_MODEPARITY|PCD_MODESDATA)

/*
@Enum Type of Cpu<nl>
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumCPUType
{
	PCDUNKNOWN	= 0,	//@Emem Undefined PCD (0)
	PCD1		= 1,	//@Emem PCD1 (1)
	PCD2		= 2,	//@Emem PCD2 (2)
	PCD3		= 3,	//@Emem PCD3 (3)
	PCD4		= 4,	//@Emem PCD4 (4)
	PCD5		= 5,	//@Emem PCD5 = PCD6.M5xx (5)
	PCD6		= 6,	//@Emem PCD6 (6)
    PCD7        = 7,    //@Emem PCD7 L-Module (7)
    PCS1        = 8,    //@Emem PCS1 Kompakt Regler (8)
	PCD7V       = 9,    //@Emem PCD7 Vaillant
	PCDMxx7		= 255,	//@Emem Siemens emulator xx7 (255)
    PCD_LAST_TYPE = PCD7V,   //@Emem Last valid PCD type
};

/*
@Enum SubType of Cpu<nl>This value is stored in the version number, from byte 3 onwards.
 It is converted from a string to an integer.<nl>Use the function <f PcdGetInternalData>
 to access the internal data.
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumCPUSubType
{
    PCDX_M1     = 1,    //@Emem M1 (1)
    PCDX_M2     = 2,    //@Emem M2 (2)
	PCDX_M3     = 3,	//@Emem M3 (3)
	PCDX_M4     = 4,	//@Emem M4 (4)
	PCDX_M5	    = 5,	//@Emem M5 (5)
    PCDX_M150   = 15,   //@Emem M150 (15)
    PCDX_M170   = 17,   //@Emem M170 (17)
    PCDX_M480   = 48,   //@Emem M480 (48)
	PCDX_T770	= 77,	//@Emem T770 (77)
    PCDX_M910   = 91,   //@Emem M910 (91)
};


#define PCD_FIRST_CPU 0			//@def PCD_FIRST_CPU | First Cpu number (0).
#define PCD_LAST_CPU  6			//@def PCD_LAST_CPU | Last Cpu number (6).

#define PCD_FIRST_STATION 0		//@def PCD_FIRST_STATION | First Station number (0).
#define PCD_LAST_STATION 254	//@def PCD_LAST_STATION | Last Station number (reserved) (254).
#define PCD_MAXSTATION (PCD_LAST_STATION-(PCD_FIRST_STATION+1))	//@def PCD_MAXSTATION | Largest valid station number.
#define PCD_MAXFDLADD 126		//@def PCD_MAXFDLADD | Last PROFI-S-BUS station number (126)
/*
@Enum Maximum values for element numbers (PCD6)<nl>
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumMaxValues
{
	PCD_MAXCPU     = 6,			//@Emem Last CPU number (6)
	PCD_MAXREG     = 4095,		//@Emem Register (4095)
	PCD_MAXREGEX   = 16383,		//@Emem Extended Registers (16383)
	PCD_MAXIO      = 8191,		//@Emem Input and Output (8191)
	PCD_MAXFLAG    = 8191,		//@Emem Flag (8191)
    PCD_MAXFLAGEX  = 16383,     //@Emem Extended Flags (16K - 1)
	PCD_MAXTMR     = 1599,		//@Emem Timer (1599)
	PCD_MAXCTR     = 1599,		//@Emem Counter (1599)
	PCD_MAXTC      = 1599,		//@Emem Timer or Counter (1599)
	PCD_MAXM       = 1,			//@Emem Memory flag (1)
	PCD_MAXK       = 16383,		//@Emem Constant K (16383)
	PCD_MAXFB      = 1999,		//@Emem Function Block (1999)
	PCD_MAXPB      = 999,		//@Emem Program Block (999)
	PCD_MAXSTR     = 5999,		//@Emem ISTep, STep and TRansition (5999)
	PCD_MAXSB      = 95,		//@Emem Sequential Block (95)
	PCD_MAXPARM    = 255,		//@Emem FB Parameter list length (255)
	PCD_MAXCOB     = 31,		//@Emem Cyclic Orgainisation Block (31)
	PCD_MAXXOB     = 63,		//@Emem Exception Organisation Block (63)
	PCD_MAXSEMA    = 99,		//@Emem Semaphore (99)
	PCD_MAXTEST    = 777,		//@Emem Test (777)
	PCD_MAXADDS    = 999999,	//@Emem Program line number (999999)
	PCD_MAXDB1     = 3999,		//@Emem Data Block number (3999)
	PCD_MAXDB      = 8191,		//@Emem Data Block in extended memory (8191)
	PCD_MAXDBLEN1  = 383,		//@Emem Max. items in DB 0..3999: 0..382 (383)
	PCD_MAXDBLEN   = 16384,		//@Emem Max. items in DB 4000..7999: 0..16383 (16384)
	PCD_MAXTEXT1   = 3999,		//@Emem Text (3999)
	PCD_MAXTEXT    = 8191,		//@Emem Text in extended memory (8191)
	PCD_MAXTEXTLEN = 65534,		//@Emem Max. size of text (EXCLUDING NUL) (65534)
	PCD_MAXTEXTLEN1= 3072,		//@Emem Max. size of text (EXCLUDING NUL) (3072)
	PCD_MAXDEPTH   = 32,		//@Emem Max. block call nesting depth (32)
};

#define PCD_CONNCPU (PCD_MAXCPU+1)	//@def PCD_CONNCPU | connected CPU (7)
#define PCD_ALLCPUS (PCD_MAXCPU+2)	//@def PCD_ALLCPUS | Run/Stop/Restart ALL CPUs (8)

/*
@Enum Count of elements.
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumCountValues
{
	PCD_VERSIONSIZE  = 9,	//@Emem Size of version string without NUL end of string. (9)
	PCD_P800VERSSIZE = 8,	//@Emem Size of P800 version string without NUL end of string. (8)
	PCD_MAXBAUDRATE  = 12,	//@Emem Number of baudrate. (12)
	PCD_MAXPROTOCOL  = 2,	//@Emem Number of protocol P800-SBUS. (2)
	PCD_MAXRETRY     = 5,	//@Emem Number of retry. (5)
	PCD_LANSIZE      = 9,	//@Emem Size of LAN string without NUL end of string. (9)
    PCD_PROGNAMESIZE = 8,   //@Emem Size of program name in PCD's header
};

///////////////////////////////////////////////////////////////////////
// Callback structure
/*
@Enum enumComCallback values.
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
@xref <l Callback>, <t enumProgressAction>
*/
enum enumComCallback
{
	PCD_CALLBACK_OPEN = 0,		//@Emem Title for the dialog box (0)
	PCD_CALLBACK_TEXT,		    //@Emem Text for the dialog box (1)
	PCD_CALLBACK_PROGRESS,	    //@Emem Progress informations (2)
	PCD_CALLBACK_ASKCANCEL,	    //@Emem Ask for user cancel progress (3)
	PCD_CALLBACK_CLOSE,		    //@Emem Close the dialog box (4)
	PCD_CALLBACK_ENABLESTOP,    //@Emem Enable/Disable stop button (5)
    PCD_CALLBACK_MSGOK,         //@Emem Ask for user OK (6)
    PCD_CALLBACK_MSGOKCANCEL,   //@Emem Ask for user OK/Cancel (7)
    PCD_CALLBACK_MSGBUTTON,     //@Emem Ask for MSGxxx dialog button (8)
	PCD_CALLBACK_MSGYESNO,		//@Emem Ask for user YES/NO (9)
	PCD_CALLBACK_PROGRESS_TXT,	//@Emem Textual Progress informations (10)
	PCD_CALLBACK_CHOICEOPEN,	//@Emem Open a simple text choice dialog box (11)
	PCD_CALLBACK_TEXTCHOICE,	//@Emem Display text for user selection (12)
	PCD_CALLBACK_MSGCHOICEID,	//@Emem Query the selected index (first index = 100) (13)
};

#define PCD_CALLBACK_CHOICE_ID 100		//@def PCD_CALLBACK_CHOICE_ID | first index from PCD_CALLBACK_MSGCHOICEID
/*
@Enum Values for <f PcdTextType>
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumTextType
{
	PCD_TEXTTYPE_UNDEF = 0,		//@Emem Text undefined (0)
	PCD_TEXTTYPE_TEXT = 1,		//@Emem Standard text (1)
	PCD_TEXTTYPE_LAN = 2,		//@Emem Encoded binary LAN text (2)
	PCD_TEXTTYPE_ILAN = 3,		//@Emem Encoded indirect LAN text (3)
	PCD_TEXTTYPE_DB = 4,		//@Emem Encoded binary data block (4)
};

/*
@enum Values for <f PcdGetFwFeatures>
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumFwFeature {
    PCD_FEATURE_EXTENDEDHEADER = 0x0001,  //@Emem Extended header (0x0001)
    PCD_FEATURE_BREAKPARITY    = 0x0002,  //@Emem S-Bus break/parity signalling (0x0002)
    PCD_FEATURE_DATAMODE       = 0x0004,  //@Emem S-Bus data mode (0x0004)
    PCD_FEATURE_WARMSTART      = 0x0008,  //@Emem Warm start instead of cold (0x0008)
    PCD_FEATURE_GATEWAY        = 0x0010,  //@Emem Gateway master port (0x0010)
    PCD_FEATURE_SYSTEMINFO     = 0x0020,  //@Emem Supports <f PcdRdSystemInfo> (0x0020)
    PCD_FEATURE_EXTENDEDSFC    = 0x0040,  //@Emem Supports SB 32..95 and ST/TR 2000..6000 (0x0040)
    PCD_FEATURE_EXTENDEDREGS   = 0x0080,  //@Emem Supports Registers 4096..16383 (0x0080)
};

/*
@enum Values for <f PcdFlashErase>
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumFlashErase {
    PCD_FLASH_ERASE_ALL        = 0,     //@emem erase all (0)
    PCD_FLASH_ERASE_HEADER     = 1,     //@emem erase only the header (1)
    PCD_FLASH_ERASE_CPUPROG    = 2,     //@emem add CPU number to this value (2 to 8)
    PCD_FLASH_ERASE_ALLPROGS   = 9,     //@emem erase programs for all CPU's (9
    PCD_FLASH_SET_READ_MODE    = 10,    //@emem set flash in read mode (10)
    PCD_FLASH_LAST             = 10,    //@emem last PCD_FLASH_xxxx value (10)
};

#define ENTRYSIZE  26   // Bytes per entry in PCD memory
#define MAXENTRIES 32   // Maximum number of entries

/*
@Enum Control Flags for <f PcdConnectChannel><nl>
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumPcdConnectFlags {
    PCD_CHAN_NOFLAG         = 0x00000000,  // @Emem default (0x00000000)
    PCD_CHAN_NOFAILDLG      = 0x00000001,  // @Emem Don't display fail dialog box (0x00000001)
    PCD_CHAN_NOHANGUPONFAIL = 0x00000002,  // @Emem Don't hang up if connect fails (0x00000002)
    PCD_CHAN_NOPASSWORD     = 0x00000004,  // @Emem No enter password dialog box (0x00000004)
    PCD_CHAN_PUMPMSG        = 0x00000008,  // @Emem Pump messages waiting for response. Be carefull for reentrency! (0x00000008)
    PCD_CHAN_FORCEPGU9600   = 0x00000010,  // @Emem Force PGU speed to 9600 bauds (0x00000010)
	PCD_CHAN_STRICT         = 0x00000020,  // @Emem Reject telegrams with noisy data (0x00000020)
    PCD_CHAN_SKIPNOISE      = 0x00000040,  // @Emem When noisy data ignore first part (0x00000040)
    PCD_CHAN_NOUSBSELECT    = 0x00000080,  // @Emem Do not open select USB dialog when conflict (0x00000080)
    PCD_CHAN_USECALLBACK    = 0x00010000,  // @Emem Use the callback mechanisn for notifications (0x00010000)
    PCD_CHAN_USEFIRSTUSB    = 0x00020000,  // @Emem When connecting on USB with zero handle, automatically select 1st USB device connected (0x00020000)
};

/*
@Enum Protocol level from reading internal data <f PcdCheckInternalData>
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumProtocolLevel
{
    PCD_LEVEL_INVALID = 0,      //@emem Internal data has not been updated (0)
    PCD_LEVEL_REDUCED = 1,      //@emem Protocol is reduced or level 1 (1)
    PCD_LEVEL_FULL	  = 2,      //@emem Protocol is full (2)
	PCD_LEVEL_FORCE   = 0x80000000,	//@emem When set in <f CheckInternalData>, forces the internal data to be re-read (0x80000000)
};

/*
@Enum Progress callback action code for progress in download
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
@xref <l Callback>, <f PcdDownload>, <t enumComCallback>
*/
enum enumProgressAction
{
    PCD_CALLBACK_ACTION_INIT  = 0,   //@emem Initialize progress dialog box (0)
    PCD_CALLBACK_ACTION_CODE  = 1,   //@emem Process code segment (1)
    PCD_CALLBACK_ACTION_TEXT  = 2,   //@emem Process text segment (2)
    PCD_CALLBACK_ACTION_EXT   = 3,   //@emem Process extension memory segment (3)
    PCD_CALLBACK_ACTION_EMI   = 4,   //@emem Process initialisation segment (4)
    PCD_CALLBACK_ACTION_REST  = 5,   //@emem Process PCD restart (5)
    PCD_CALLBACK_ACTION_FLASH = 6,   //@emem Copy User program to flash (6)
    PCD_CALLBACK_ACTION_NONE  = 7,   //@emem Special display (not used in download) (7)
	PCD_CALLBACK_ACTION_SQL   = 8,	 //@emem Process SQL segment (8)
	PCD_CALLBACK_ACTION_FLASHERASE = 9, //@emem Procees flash erasing (9)
};

/*
@Enum Init value for PCD header <t PCDHEADER>
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumInitValues
{
    PCD_INIT_RAM        = 0xA55A,   //@emem Pcd has RAM (0xA55A)
    PCD_INIT_RAMPW      = 0xA956,   //@emem Pcd has RAM and password protection (0xA956)

    PCD_INIT_EPROM      = 0x5AA5,   //@emem Pcd has EPROM (0x5AA5)
    PCD_INIT_EPROMPW    = 0x56A9,   //@emem Pcd has EPROM and password protection (0x56A9)

    PCD_INIT_FLASH      = 0xA5A5,   //@emem Pcd has FLASH (0xA5A5)
    PCD_INIT_FLASHPW    = 0x659A,   //@emem Pcd has FLASH and password protection (0x659A)
};

/*
@Enum Code values for the firware download <f PcdFwDownload>
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumCodeSysChange
{
    PCD_SYSCHG_SBUS    = 0,         //@emem Change Pgu S-Bus communication (0)
};

/*
@Enum Code values for <f PcdFwDownload>
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumSbusTypeToChange
{
    PCD_CHG_SBUS_BAUDRATE   = 0x01, //@emem Change baurate (110 .. 115200) (0x01)
    PCD_CHG_SBUS_DATALEN    = 0x02, //@emem Change data length (7, 8) (0x02)
    PCD_CHG_SBUS_STOP       = 0x04, //@emem Change stop bits (1, 2) (0x04)
    PCD_CHG_SBUS_PARITY     = 0x08, //@emem Change parity ('N', 'O', 'E') (0x08)
    PCD_CHG_SBUS_MODE       = 0x10, //@emem Change mode (break, parity, data) (0x10)
};

/*
@Enum Code values for the firmware download <f PcdFwDownload>
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumCodeFwDownload
{
    PCD_FWDNLD_START			= 0,    //@emem Start firmware download (0)
    PCD_FWDNLD_DATA				= 1,    //@emem Write firmware data (1)
    PCD_FWDNLD_END				= 2,    //@emem End of firmware download (2)
    PCD_FWDNLD_BAUDRATE			= 3,    //@emem Try setting the baudrate to the passed value (3)
    PCD_FWDNLD_ERASE			= 4,    //@emem Erase the flash card, reponse NAK when done (4)
    PCD_FWDNLD_FLASHCRD			= 6,    //@emem Write firmware data to flashcard (6)
    PCD_FWDNLD_STARTNEW			= 0x10, //@emem Start firmware multi download (0x10)
    PCD_FWDNLD_DATANEW			= 0x11, //@emem Write firmware data (same as 0) (0x11)
    PCD_FWDNLD_ENDNEW			= 0x12, //@emem End of firmware multi download (0x12)
	PCD_FWDNLD_HEADER			= 0x20,	//@emem Write header to FW (0x20)
	PCD_FWDNLD_CHKDNLDHEADER	= 0x21,	//@emem Check if write header is supported (0x21)
	PCD_FWDNLD_KEY				= 0x83, //@emem Test Key in OEM PLC, reponse NAK if Key not valid (0x83)
    PCD_FWDNLD_ABORT			= 0xFF, //@emem Abort the firmware download (0xFF)
};

/*
@Enum Flags for ACCU and arithmetic status,
 for <f PcdRdAccu>, <f PcdWrAccu> and <f PcdBreakpoint>
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumAccuFlags
{
    PCD_ZERO   = 0x0008,  //@Emem Zero flag (0x0008)
    PCD_NEG    = 0x0004,  //@Emem Negative flag (0x0004)
    PCD_ERROR  = 0x0002,  //@Emem Error flag (0x0002)
    PCD_ACCU   = 0x0001,  //@Emem Accumulator (0x0001)
};

/*
@Enum PCD Restarting<nl>
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumPcdRestart
{
	PCD_COLD = 0,			// @Emem Cold restart (0)
	PCD_WARM = 1,			// @Emem Warm restart (1)
};

/*
@Enum Flags for "Control" value in <f PcdDownload>, <f PcdUpload>, <f PcdDelete>,
 <f PcdDownloadBlock> and <f PcdBuesDownload> OR'd together<nl><nl>
 Additional flags for <f PcdDelete> and <f PcdWrAllocation><nl>
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumPcdDownLoadFlags
{
    PCD_CODE            = 0x00000001,	  // @Emem Code segment (0x00000001)
    PCD_TEXT            = 0x00000002,	  // @Emem Text/DB segment (0x00000002)
    PCD_EXTMEM          = 0x00000004,	  // @Emem Extension memory (0x00000004)
    PCD_IB              = 0x00000008,	  // @Emem IB segment (0x00000008)
    PCD_VERIFY  		= 0x00000010,	  // @Emem Verify all memory writes (0x00000010)
    PCD_ERRDLG  		= 0x00000020,	  // @Emem Display error message dialog box if error (0x00000020)
    PCD_STOPDLG 		= 0x00000040,	  // @Emem Display "CPU's will be stopped" dialog box (0x00000040)
    PCD_OKEND   		= 0x00000080,	  // @Emem "OK" button when up/download complete (0x00000080)
    PCD_OVRDLG  		= 0x00000100,	  // @Emem Display "overwrite file" dialog box (0x00000100)
    PCD_NOSTOP  		= 0x00000200,	  // @Emem Do not stop cpu before downloading (0x00000200)
    PCD_NORUN   		= 0x00000400,	  // @Emem Do not run Cpu after downloading (0x00000400)
    PCD_EMISEG_NONE     = 0x00000800,	  // @Emem No extension memory init segment (0x00000800)
    PCD_EMISEG_COMPACT  = 0x00001000,	  // @Emem Compact extension memory init segment (0x00001000)
    PCD_AUTOMEMALLOC    = 0x00002000,     // @Emem Automatic memory allocation (0x00002000)
    PCD_NOCLROUTPUTS    = 0x00004000,	  // @Emem Don't clear outputs (0x00004000)
    PCD_WARMSTART       = 0x00008000,	  // @Emem Do restart warm (0x00008000)
    PCD_USECALLBACK     = 0x00010000,     // @Emem Use the callback mechanisn for notification (0x00010000)
    PCD_BACKUPEXTENMEMTOFLASH = 0x00020000, // @Emem Backup extension memory to FlashCard (0x00020000)
    PCD_COPYTOFLASH     = 0x00040000,     // @Emem Copy user program to flah on M170 after sucessfull download
                                          //       This only applies whan downloading ALLSEGS (0x00040000)
    PCD_NORESTART       = 0x00080000,     // @Emem Do not perform a restart after downloading the user program

    PCD_ALLSEGS = (PCD_CODE|PCD_TEXT|PCD_EXTMEM),	// @Emem  CODE, TEXT and EXTMEM segments
    PCD_DEFAULT = (PCD_ALLSEGS|PCD_ERRDLG|PCD_STOPDLG|PCD_OKEND|PCD_OVRDLG), //@Emem ALLSEGS, ERRDLG, STOPDLG, OKEND and OVRDLG flags

	PCD_FL_KEEPSUBSYS	= 0x10000000,	  // @Emem Set when subsystem have to be preserved  (0x10000000)
};

/*
@enum condition types for function <f PcdWrSystemBuffer> and <f PcdRdSystemBuffer>
@Comm Do not use the value of those constants but there symbolic name to avoid
 problems in future releases.<nl>The values are only indicated for information purposes.
*/
enum enumTraceBuffer {
    PCD_TRACEBUFFER = 0,   //@Emem (0)
};

/*
@enum condition types for function <f PcdSyncTrace>
@Comm Do not use the value of those constants but there symbolic name to avoid
 problems in future releases.<nl>The values are only indicated for information purposes.
*/
enum enumSyncTraceCondition {
    PCD_SYNCTRACE_IL  = 0,  //@Emem IL synchronisation trace (0)
    PCD_SYNCTRACE_FUP = 1,  //@Emem Fupla probe mode (1)
};

/*
@enum Type codes for <f PcdRdSystemInfo>
@Comm Do not use the value of those constants but there symbolic name to avoid
 problems in future releases.<nl>The values are only indicated for information purposes.
 See DDPF6027.doc for details.
*/
enum enumSystemInformation {
    PCD_SYSIIMPLEMENTATION = 0, //@Emem Which function is implemented, 1 byte <t enumSysImplementation> (0)
    PCD_SYSIMEMORYSIZE = 1,     //@Emem Memory size (6 bytes) (1)
    PCD_SYSITRACEBUFSIZE = 2,   //@Emem Size of trace buffer (2 bytes) (2)
    PCD_SYSIEEPROMINFO = 3,     //@Emem System information in EEPROM on port B1 (16 bytes) (3)
    PCD_SYSIEEPROMINFOB2 = 4,   //@Emem System information in EEPROM on port B2 (16 bytes) (4)
    PCD_SYSICHGPGUSPEED = 5,    //@Emem Change baudrate for S-Bus PGU (4 bytes min.) (5)
    PCD_SYSIONBOARDINFO = 6,    //@Emem Onboard information (6)
    PCD_SYSISAIANTINFO = 7,     //@Emem SAIA_NT information (7)
    PCD_SYSISLOTINFO = 8,       //@Emem Slot information (8)
	PCD_SYSEXTSTATUS = 10,		//@Emem Extended status for going online (10)

};

/*
@enum Flags for <f PcdRdSystemInfo> PCD_SYSIIMPLEMENTATION return value.
 These bits indicate which firmware features are supported.<nl>
PcdRdSystemInfo(... , PCD_SYSIIMPLEMENTATION, ...) currently returns only one byte,
 but could be extended to return more bytes later.
*/
enum enumSysImplementation
{
    // byte 0
    PCD_SYSIMP_UNKNOWN1     = 0x01, //@Emem Unknown?
    PCD_SYSIMP_MEMORYSIZE   = 0x02, //@Emem PCD_SYSIMEMORYSIZE supported
    PCD_SYSIMP_TRACEBUFFER  = 0x04, //@Emem PCD_SYSITRACEBUFSIZE supported
    PCD_SYSIMP_EEPROMINFO   = 0x08, //@Emem PCD_SYSIEEPROMINFO supported
    PCD_SYSIMP_EEPROMINFOB2 = 0x10, //@Emem PCD_SYSIEEPROMINFOB2 supported
};

/*
@enum Type code for <f PcdFlashcard>, <f PcdCopyToFlashEx>
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumFlashcardType {
    PCD_FLASHCARD_TORAM = 0,       //@Emem Copy the user program from flashcard to ram (0)
    PCD_FLASHCARD_TOFLASH = 1,     //@Emem Copy the user program from ram to flashcard (1)
    PCD_FLASHCARD_READLINE = 2,    //@Emem Read line from flashcard (2)
    PCD_FLASHCARD_FWTOFLASH = 4,   //@Emem Copy FW to Flashcard (4)
    PCD_FLASHCARD_BACKUP = 5,      //@Emem Backup user program to flash with options (5)
    PCD_FLASHCARD_RESTORE = 6,     //@Emem Restore user program from flash with options (6)
    PCD_FLASHCARD_POLL = 255,      //@Emem Test if flashcard command is finished (255)
};

/*
@enum Options for <f PcdCopyToFlashEx>
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumFlashCardOptions {
    PCD_FLASHCARD_OPT_ONBRD_BKP    = 0x00000001,   //@Emem Do an Onboard Backup/Restore (1)
    PCD_FLASHCARD_OPT_FILE_BKP     = 0x00000002,   //@Emem Do a File Backup/Restore (2)
    PCD_FLASHCARD_OPT_IMAGE_BKP    = 0x00000004,   //@Emem Do an Image Backup/Restore (4)
    PCD_FLASHCARD_OPT_MEDIA_BKP    = 0x00000008,   //@Emem Do a Media Backup/Restore (8)
    PCD_FLASHCARD_OPT_CONFIG_BKP   = 0x00000010,   //@Emem Do a Configuration file Backup/Restore (16)
    PCD_FLASHCARD_OPT_WEB_BKP      = 0x00000020,   //@Emem Do a Web file Backup/Restore (32)
    PCD_FLASHCARD_OPT_CLR_BKP      = 0x00000100,   //@Emem Delete backups from same flash (256)
    PCD_FLASHCARD_OPT_CLRALL_BKP   = 0x00000200,   //@Emem Delete all backups on all flash (512)
    PCD_FLASHCARD_OPT_CLRONBOARD_BKP = 0x00000400, //@Emem Delete onboard backup (1024)
};

/*
@enum Types of blocks for the structure <t PCDMULTIBLOCKADDRESS>
*/
enum enumPcdBlockType {
    PCD_BLOCK_COB   = 0,    //@Emem COB (0)
    PCD_BLOCK_XOB   = 1,    //@Emem XOB (1)
    PCD_BLOCK_PB    = 2,    //@Emem PB (2)
    PCD_BLOCK_FB    = 3,    //@Emem FB (3)
    PCD_BLOCK_ST    = 4,    //@Emem ST Only possible if SB structure does not change (4)
    PCD_BLOCK_TR    = 5,    //@Emem TR Only possible if SB structure does not change (5)
    PCD_BLOCK_TEXT  = 6,    //@Emem TEXT (6)
    PCD_BLOCK_DB    = 7,    //@Emem DB (7)
    PCD_BLOCK_SB    = 8,    //@Emem SB (not possible for the moment) (8)
    PCD_BLOCK_DBX   = 10,   //@Emem DBX (10)
};

/*
@enum Type of feature for Extended header <t PCDEXTHDR2>
*/
enum enumPcdExtHdrFeatureType {
    PCD_EXTHDR_MODEM_NAME  = 1,     //@Emem Record is name of modem (1)
    PCD_EXTHDR_MODEM_RESET = 2,     //@Emem Record is modem reset string (2)
    PCD_EXTHDR_MODEM_INIT  = 3,     //@Emem Record is modem initialisation string (3)
	PCD_EXTHDR_MODEM	   = 4,		//@Emem Record is S-Bus modem (4)
    PCD_EXTHDR_GATEWAY     = 10,    //@Emem Record is gateway configuration (10)
	PCD_EXTHDR_TCPIPGWAY   = 11,	//@Emem Record is TCP/IP gateway (11)
	PCD_EXTHDR_PROFISBUSGWAY = 12,	//@Emem Record is Profi-S-Bus gateway (12)
    PCD_EXTHDR_GWAYSTNRANGE = 13,   //@Emem Record is serial gateway station range (13)
    PCD_EXTHDR_IP          = 20,    //@Emem Record is IP card 1 configuration (20)
    PCD_EXTHDR_IP2         = 21,    //@Emem Record is IP card 2 configuration (21)
	PCD_EXTHDR_PROFISBUS0  = 30,	//@Emem Record is Slot B1 Profi-S-Bus (30)
	PCD_EXTHDR_PROFISBUS1  = 31,	//@Emem Record is Slot B2 Profi-S-Bus (31)
	PCD_EXTHDR_PROFISBUS2  = 32,	//@Emem Record is Onboard Profi-S-Bus (32)
    PCD_EXTHDR_PCDOPTFLAGS = 40,    //@Emem Record is PCD Option Flags (40)
    PCD_EXTHDR_IPDHCP1     = 50,    //@Emem DHCP IP module 1 (50)
    PCD_EXTHDR_IPDHCP2     = 51,    //@Emem DHCP IP moudle 2 (51)
};

/*
@enum State of the password protection <t PCDINTERNALDATA>
*/
enum enumPcdPasswordState {
    PCD_PASSWORD_NONE = 0,          //@Emem Pcd is not password protected (0)
    PCD_PASSWORD_ENABLE = 1,        //@Emem Pcd is password protected and active (1)
    PCD_PASSWORD_DISABLE = 2,       //@Emem Pcd is password protected but inactive (2)
};

/*
@Enum channel type in registry
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum ChannelTypesTag
{
    CHANNEL_PGU			= 1,    //@Emem PGU channel (1)
    CHANNEL_SBUS		= 2,    //@Emem S-Bus channel (2)
    CHANNEL_PC104		= 3,    //@Emem PC104, dual port memory channel (3)
    CHANNEL_BUES		= 4,    //@Emem BUES (special channel) (4)
    CHANNEL_SBUS_MODEM	= 5,    //@Emem S-Bus Modem channel (5)
    CHANNEL_SOCKET      = 6,    //@Emem S-Bus TCP/IP socket channel (6)
	CHANNEL_SBUS_USB    = 7,	//@Emem S-Bus USB channel(7)
	CHANNEL_PROFI_SBUS  = 8,	//@Emem Profi-S-Bus channel(8)

    MAX_DEFAULT_CHANNEL = 8,    //@Emem Number of channels !
};

/*
@enum Special flags for Bues buffer behaviour
@Comm Do not use the value of those constants but there symbolic name to avoid
 Problems in future release.<nl>The value are only indicated for information purpose.
*/
enum enumBUESFLAGS
{
	BUES_RAW_MODE = 1,	//@emem When set, Bues frame are returned verbatim (1)
};

/*
@Enum Special flags for Bues connections <nl>
 Warning:<nl>Do not use the value of those constants but there symbolic name to avoid
 Problems in future release<nl>The value are only indicated for information purpose.
*/
enum tagPCDCONNFLAG
{
	PCD_FL_ISBUESPP		= 0x00000001,	//@Emem Set when main station is BuEs1++  (0x40000000)
	PCD_FL_ISBUESUP		= 0x00000002,	//@Emem Set when main station is Bues1 update  (0x20000000)
	PCD_FL_NOAUTODIAL	= 0x00000004,	//@Emem Set to disable automatic RAS dial (0x00000004)
	PCD_FL_NOHANGUP		= 0x00000008,	//@Emem Set to disable automatic RAS hang-up (0x00000008)
};

/*
@Enum Special flags for SCDlgChannelEx and for <f PcdReadOpendata><nl>
 Warning:<nl>Do not use the value of those constants but there symbolic name to avoid
 Problems in future release<nl>The value are only indicated for information purpose.
*/
enum tagCHANNELDLGEXFLAGS
{
	CHANNELEX_MERGE = 1,		//@Emem When editing the channel settings, merge the channel with
								// the original passed to the function <f PcdChannelListEx>
};

//@enum Values for PCDEXTHDR2.dwPcdOptFlags
enum tagPCDOPTFLAGS
{
    PCDOPTFLAG_RESETOUTPUTENABLE = 0x00000001,
    PCDOPTFLAG_XOB1ENABLE        = 0x00000002,
    PCDOPTFLAG_RUNHALTENABLE     = 0x00000004,
    PCDOPTFLAG_FULLRS232         = 0x00000008,
    PCDOPTFLAG_TERMINATION       = 0x00000010,
    PCDOPTFLAG_ETHERNETRIO       = 0x00000020,
};

/*
@Enum Function types for the Read/Write Block telegram <f PcdRWBlock>
*/
enum tagPCDFUNRWTYPES
{
	PCDWRITEBLOCKSTARTSTREAM	= 0x00,	//@Emem Write block start of stream (0x0)
	PCDWRITEBLOCKDATASTREAM		= 0x01,	//@Emem Write block data stream (0x1)
	PCDWRITEBLOCKENDSTREAM		= 0x02,	//@Emem Write block end of stream (0x2)
	PCDWRITEABORT				= 0x07,	//@Emem Write block abort (0x7)
	PCDWRITEBLOCKDATA			= 0x08,	//@Emem Write block data (0x8)
	PCDREADBLOCKSTARTSTREAM		= 0x10,	//@Emem Read block start of stream (0x10)
	PCDREADBLOCKDATASTREAM		= 0x11,	//@Emem Read block data stream (0x11)
	PCDREADABORT				= 0x17,	//@Emem Read block abort (0x17)
	PCDREADBLOCKDATA			= 0x18,	//@Emem Read block data (0x18)
	PCDDELETEBLOCK				= 0x20,	//@Emem Delete block (0x20)
	PCDSIZEBLOCK				= 0x21,	//@Emem Get block size (0x21)

	PCDDNLDSTARTINHLT			= 0x40,	//@Emem Download start, set PCD in HALT (0x40)
	PCDDNLDSTARTINRUN			= 0x41,	//@Emem Download start, set PCD in RUN (0x41)
	PCDDNLDSTARTCHG				= 0x42,	//@Emem Download start changed block in RUN (0x42)
	PCDDNLDENDRUN				= 0x48,	//@Emem End of download, restart in RUN (0x48)
	PCDDNLDENDSTOP				= 0x49,	//@Emem End of download, restart in STOP (0x49)
	PCDDNLDENDCHG				= 0x4A,	//@Emem End of download changed block in RUN (0x4A)
	PCDDNLDABORT				= 0x4E,	//@Emem Abort download (0x4E)
	PCDDNLDPOLL					= 0x4F	//@Emem Poll the status (0x4F)
										//@Flag 0  | Busy
										//@Flag <gt>0 | OK
										//@Flag <lt>0 | Error code
};

/*
@Enum Block types for the Read/Write Block telegram <f PcdRWBlock>
*/
enum tagPCDBLOCKRWTYPES
{
	PCDBLOCK_COB	= 0x00,		//@Emem Block is COB (0x0)
	PCDBLOCK_XOB	= 0x01,		//@Emem Block is XOB (0x1)
	PCDBLOCK_PB		= 0x02,		//@Emem Block is PB (0x2)
	PCDBLOCK_FB		= 0x03,		//@Emem Block is FB (0x3)
	PCDBLOCK_ST		= 0x04,		//@Emem Block is ST (0x4)
	PCDBLOCK_TR		= 0x05,		//@Emem Block is TR (0x5)
	PCDBLOCK_TEXT	= 0x06,		//@Emem Block is TEXT (0x6)
	PCDBLOCK_DB		= 0x07,		//@Emem Block is DB (0x7)
	PCDBLOCK_SB		= 0x08,		//@Emem Block is SB (0x8)
	PCDBLOCK_DBX	= 0x09,		//@Emem Block is DBX (0x9)
	PCDBLOCK_BAC	= 0x10,		//@Emem Block is BACnet configuration (0x10)
	PCDBLOCK_CAN	= 0x11,		//@Emem Block is CANopen configuration (0x11)
	PCDBLOCK_LON	= 0x12,		//@Emem Block is LONIP configuration (0x12)

	PCDBLOCK_CONF	= 0x20,		//@Emem Block is Configuration file (0x20)
	PCDBLOCK_FILE	= 0x21,		//@Emem Block is file (0x21)
	PCDBLOCK_WEB	= 0x22,		//@Emem Block is Web (0x22)
    PCDBLOCK_BAC14  = 0x23,     //@Emem Block is BACet configuration rev. 14 (0x23)
	PCDBLOCK_FILESY = 0x2F, 	//@Emem Block is fileSystem (0x2F)
};

/*
@Enum Block Status for the Read/Write Block telegram <f PcdRWBlock>
*/
enum tagPCDBLOCKRWSTATUS
{
	PCDBLOCKSTATUSACK		= 0x00,		//@Emem ACK  (0x0)
	PCDBLOCKSTATUSDATA		= 0x01,		//@Emem Data (0x1)
	PCDBLOCKSTATUSBUSY		= 0x02,		//@Emem Busy (0x2)
	PCDBLOCKSTATUSEND		= 0x03,		//@Emem End  (0x3)
	PCDBLOCKSTATUSEOF		= 0x04,		//@Emem Enf of block/file only for PCDREADBLOCKDATA(04)
	PCDBLOCKSTATUSNAK		= 0x80,		//@Emem NAK  (0x80 - 0xfe)
	PCDBLOCKSTATUSABORT		= 0xff,		//@Emem Abort (0xff)
};

/*
@Enum Media codes for <f PcdRdTaskData> and <f PcdWrTaskData> telegrams
*/
enum tagPCDTASKDATAMEDIATYPE
{
	PCD_TASK_FLAG		= 0x00,		//@Emem Task Flag (0x0)
	PCD_TASK_REGISTER	= 0x01,		//@Emem Task Register (0x1)
};

/*
@Enum Media codes for <f PcdRdPeriData> and <f PcdWrPeriData> telegrams
*/
enum tagPCDPERIFDATAMEDIATYPE
{
    PCD_PERI_BYTE       = 0x0,     //@Emem Peripheral Byte (0)
    PCD_PERI_WORD       = 0x1,     //@Emem Peripheral Word (1)
    PCD_PERI_DWORD      = 0x2,     //@Emem Peripheral DWord (2)
    PCD_PERI_BUFFER     = 0x3,     //@Emem Peripheral Buffer (3)
    PCD_PERI_DIAG       = 0x4,     //@Emem Peripheral Diagnostics (4)
};

/*
@Enum Media codes internal (do not use) for interface, use <t tagPCDPERIFDATAMEDIATYPE>
*/
enum tagPCDPERIFDATAMEDIACODE
{
    PCD_READ_PERI_BYTE       = 0x80,     //@Emem Read Peripheral Byte (0x80)
    PCD_READ_PERI_WORD       = 0x81,     //@Emem Read Peripheral Word (0x81)
    PCD_READ_PERI_DWORD      = 0x82,     //@Emem Read Peripheral DWord (0x82)
    PCD_READ_PERI_BUFFER     = 0x83,     //@Emem Read Peripheral Buffer (0x83)
    PCD_WRITE_PERI_BYTE      = 0x84,     //@Emem Write Peripheral Byte (0x84)
    PCD_WRITE_PERI_WORD      = 0x85,     //@Emem Write Peripheral Word (0x85)
    PCD_WRITE_PERI_DWORD     = 0x86,     //@Emem Write Peripheral DWord (0x86)
    PCD_WRITE_PERI_BUFFER    = 0x87,     //@Emem Write Peripheral Buffer (0x87)
    PCD_READ_PERI_DIAG       = 0x88,     //@Emem Read Peripheral Diagnostics (0x88)
};

/*
@Enum Port status code for <f PcdRdExtStatus> telegram
*/
enum tagPCDPORTEXTSTATUS
{
	PCD_EPS_REDUCED		= 0,	//@Emem PCD is in reduced protocol (0)
	PCD_EPS_PASSWD		= 1,	//@Emem PCD is password protexted (1)
	PCD_EPS_FULL		= 2,	//@Emem PCD is in full protocol (2)
};

/*
@Enum PCD System type code for <f PcdRdExtStatus> telegram
*/
enum tagPCDPORTEXTSYSTYPE
{
	PCD_EST_UNKN		= 0,	//@Emem PCD is unknown (0)
	PCD_EST_CLASSIC		= 1,	//@Emem PCD is Classic (1)
	PCD_EST_XX7			= 2,	//@Emem PCD is XX7 (2)
	PCD_EST_MB			= 3,	//@Emem Device is MB (3)
	PCD_EST_EL			= 4,	//@Emem Device is Electroline (4)
};

/***** Structures *****/

/*
@type PCDCONN | Handle to connection informations.
@comm This handle is passed by value to most of the Pcdxxx functions,
 except for functions <f PcdDialDialog>, <f PcdConnectChannel> and
 <f PcdComOpen> where the address of the handler is needed.
*/
typedef DWORD PCDCONN;

// Open and connection.

#define PCD_PHONESIZE 64		//@def PCD_PHONESIZE | Size of phone number string (64)
#define PCD_CHANNELSIZE 64		//@def PCD_CHANNELSIZE | Size of channel string (64)
#define PCD_DEVICESIZE 64		//@def PCD_DEVICESIZE | Size of device name (64)
#define PCD_SECTIONSIZE 64		//@def PCD_SECTIONSIZE | Size of section string (64)
#define PCD_LOCSIZE 64			//@def PCD_LOCSIZE | Size of location name (64)
#define PCD_FILE_MAXPATH 260	//@def PCD_FILE_MAXPATH | Size of file path name (260)

/*
@Struct REQSTATION | Station information. The data in this structure identifies the station and
 is passed along each request to the driver
*/
typedef struct tagREQSTATION
{
    WORD    SbusStation;    //@Field S-Bus station number
	WORD	FdlStation;		//@Field Profi-S-Bus station number
    WORD    TcpPort;        //@Field Tcp port number
    DWORD   IpAddress;      //@Field Ip address<nl>format "1.2.3.4" = 0x01020304
} REQSTATION;

/*
@Struct PCDOPENDATA | PCD connection control structure.
@devnote The size of this structure should be padded to 512 bytes
 by adjusting the <e PCDOPENDATA.Reserved> member.<nl>
 The structure member alignement is 8 bytes!
@Xref <f PcdGetChannelSetup>, <f PcdDialDialog>, <f PcdConnectChannel>,
 <f PcdRdChanSetupFromIni>, <f PcdWrChanSetupToIni>, <f PcdGetOpenData>,
 <f PcdStatusDialog>, <f PcdComOpen>, <f PcdGetChannelSetupEx>,
 <f PcdConnectionDialog>, <f PcdConnectionDialogEx>.
*/
typedef struct tagPCDOPENDATA
{
	SHORT	Port;						//@Field Port number TAPI,RAS,COMx,SOCKET <t enumComPort>
	SHORT	Device;						//@Field COM port number or Socket Port.

	BOOL	bPguMode;					//@Field TRUE=using PGU mode (S-Bus/P800)
	DWORD	SbusMode;					//@Field S-Bus mode: <t enumSBusModes>
    DWORD	Protocol;					//@Field Type of protocol, <t enumProtocol>

	DWORD	BaudRate;					//@Field 110..115200
	DWORD	TsDelay;					//@Field S-Bus training sequence delay, mS
	DWORD	TnDelay;					//@Field S-Bus turnaround time, mS
	DWORD	Timeout;					//@Field S-Bus timeout in mS
	DWORD	BreakLen;					//@Field S-Bus break length in chars, PCD_BREAK mode
	DWORD	UartFifoLen;				//@Field Number of characters in UART Fifo buffer (to wait for RTS)
                                        // if this value is 0, the driver will handle the togging of RTS
                                        // automatically (NT/2000 only)
	BOOL	bDontClose;					//@Field Do not close the port.
	BOOL	bPortOpen;					//@Field The port is open.
    BOOL    bConnected;                 //@Field The port is connected with the function <f PcdConnectChannel>

    // connection
	DWORD	dwType;						//@Field Channel type: <t ChannelTypesTag>.
	CHAR	Channel[PCD_CHANNELSIZE];	//@Field Channel name 'PGU'. <t PCD_CHANNELSIZE>
	CHAR	Section[PCD_SECTIONSIZE];	//@Field Name of the section in INI file or Registry <t PCD_SECTIONSIZE>
	DWORD	ModeToTry;					//@Field Mode to connect: <t enumSBusModesToTry>
	DWORD	Cpu;						//@Field CPU number: 0..6
	BOOL	bAutoStn;					//@Field TRUE=send "read S-Bus station" telegram
	DWORD	Retry;						//@Field Retry count, default = 3

    // for TAPI, RAS
    CHAR	DeviceName[PCD_DEVICESIZE];	//@Field TAPI, RAS Modem name <t PCD_DEVICESIZE>

    // TAPI connection
	BOOL	bUseModem;					//@Field Use dialing (TAPI modem).
	BOOL	bAutoAnswer;				//@Field Open the TAPI port in AutoAnswer mode.
	CHAR	PhoneNumber[PCD_PHONESIZE];	//@Field Phone number for TAPI or RAS dialing. <t PCD_PHONESIZE>
	DWORD	CountryCode;				//@Field Country code (Switzerland 41).
	DWORD	AreaCode;					//@Field Area code (Morat 26).
	CHAR	Location[PCD_LOCSIZE];		//@Field Location name. <t PCD_LOCSIZE>
	BOOL	bUseDialing;				//@Field Use dialing (translate phone number).
	DWORD	DialRetry;                  //@Field Number of retry when dialing.

	// Bues Connection
	BOOL	bBues;					    //@Field Bues flag

	// password dialog box parent window
	HWND hPWDlgParentWnd;				//@Field Parent window for password dialog box <t PcdSetParentWnd>.

    REQSTATION ReqStation;              //@Field Complete station number information <t REQSTATION>
	SHORT	SrcSap;						//@Field Source service access point for Profi-S-Bus communication
	SHORT	DstSap;						//@Field Destination service access point for Profi-S-Bus communication
	SHORT	BDstSap;					//@Field Broadcast Destination service access point for Profi-S-Bus communication

    // JPE 09.10.00 14:53
    BYTE    Reserved[70];               //@Field Reserved for future extension and padding

} PCDOPENDATA;

//@type LPOPENDATA | Pointer to <t PCDOPENDATA>
typedef PCDOPENDATA* LPOPENDATA;
//@type LPCOPENDATA | Constant pointer to <t PCDOPENDATA>
typedef const PCDOPENDATA* LPCOPENDATA;

/*
@Struct PCDINTERNALDATA | PCD internal data structure.
 This structure can be accessed by <f PcdGetInternalData>.
@devnote The size of this structure should be padded to 96 bytes
 by adjusting the <e PCDINTERNALDATA.Reserved> member
*/
typedef struct tagPCDINTERNALDATA
{
	DWORD	nLevel;	    	    //@Field see <t enumProtocolLevel>

    // level1 internal data
    DWORD	Cpu;	            //@Field Connected CPU
	DWORD	CpuType;	        //@Field Connected CPU type: <t enumCPUType>
    DWORD   CpuSubType;         //@Field integer at &szVersion[3] <t enumCPUSubType>
	DWORD   CpuStatus;    	    //@Field CPU status: <t enumPCDStatus>

    // full protocol
    DWORD	BaseAdds;	        //@Field Base address of user memory (68000)

    DWORD   CodeStart;    	    //@Field Offset to 1st user program line
	DWORD   CodeEnd;	    	//@Field Offset to last user program line
	DWORD   CodeLast;	    	//@Field Offset to last line of code seg
	DWORD   UserEnd;	    	//@Field Last user program line number (0=1st)
	DWORD   UserLast;	    	//@Field Last user program line in code segment

    BOOL	bP800OldFW;	    	//@Field TRUE=can't use new P800 telegrams
	BOOL    bEPROM;	    	    //@Field TRUE if EPROM memory (FALSE if RAM)
	BOOL    bFlashEPROM;  	    //@Field TRUE if FLASH EPROM memory (FALSE if RAM)
	BOOL    bHasExt;	    	//@Field TRUE if extension memory present

    // JPE 09.10.00 14:53
    DWORD   MaxCpu;             //@Field Maximum number of CPU supported by this PCD
    DWORD   CpuCount;           //@Field Number of installed CPU

    // JPE 27.09.01
    DWORD   PasswordState;      //@Field State of the password <t enumPcdPasswordState>

	// JPE 24.08.03
    DWORD   BuesFlags;			//@Field Set to allow Bues special functionning <t tagPCDCONNFLAG>

    BYTE    Reserved[20];       //@Field Reserved for future extension and padding

} PCDINTERNALDATA;

//@type LPINTERNALDATA | Pointer to <t PCDINTERNALDATA>
typedef PCDINTERNALDATA* LPINTERNALDATA;

//@struct PCDCOMSTATISTICS | Structure for <t PcdGetComStatistics>.
typedef struct tagPCDCOMSTATISTICS
{
	unsigned long retryCount;		//@Field Number of corrected errors
	unsigned long errorCount;		//@Field Number of fatal errors
	unsigned long requestCount;		//@Field Number of requests
	unsigned long lastStatusCode;	//@Field Last error code
} PCDCOMSTATISTICS;

//@type LPCOMSTATISTICS | Pointer to <t PCDCOMSTATISTICS>
typedef PCDCOMSTATISTICS* LPCOMSTATISTICS;

//@struct PCDPHONEBOOK | Structure for Phone Book, see <t PcdPhonebookDialog>
//@Comm On return, the FileName member if changed to the actual phonebook file name.
typedef struct tagPCDPHONEBOOK {
	CHAR FileName[PCD_FILE_MAXPATH];		//@Field Name of the file containg the phone book (Empty = search)
	CHAR PhoneNumber[PCD_PHONESIZE];		//@Field Telephone number
	WORD Station;							//@Field Station number
	WORD Cpu;								//@Field Cpu number
	WORD Retries;							//@Field Number of retries
	WORD Auto;								//@Field Automatically determine station number
	WORD DontClose;							//@Field Don't close port when done
} PCDPHONEBOOK;

//@type LPPHONEBOOK | Pointer to <t PCDPHONEBOOK>
typedef PCDPHONEBOOK* LPPHONEBOOK;

///////////////////////////////////////////////////////////////////////
// Public Interface structures

#pragma pack(push,1)        // Byte alignement !!!

/*@Struct PCDCLOCK | PCD internal hardware clock */
typedef struct tagPCDCLOCK
{
    WORD WeekOfYear;    // @Field 1..53
    WORD DayOfWeek;     // @Field 1=Monday..7=Sunday
    union
    {
        WORD DateTime[6];
        struct
        {
            WORD Year;      //@Field 00..99 (50..99=1950..1999, 00..49=2000..2049)
            WORD Month;     //@Field 1..12
            WORD Day;       //@Field 1..31
            WORD Hour;      //@Field 0..23
            WORD Minute;    //@Field 0..59
            WORD Second;    //@Field 0..59
        };
    };
} PCDCLOCK;

/*
@Struct PCDHEADER | Memory header for one CPU
*/
typedef struct tagPCDHEADER
{
	BYTE  ProgName[8];	//@Field Program name (from PCD file name)

	DWORD CodeStart;	//@Field Code start line (1 line==4 bytes)
	DWORD CodeUsed;		//@Field Code used in lines
	DWORD CodeFree;		//@Field Code free space in lines
	DWORD TextStart;	//@Field Text start line
	DWORD TextUsed;		//@Field Text used in bytes
	DWORD TextFree;		//@Field Text free space in bytes

	WORD  Init;			//@Field Header init word <t enumInitValues>
	BYTE  Modified;		//@Field Modification indicator, 0=not modified
	BYTE  Feature;		//@Field Feature indicator byte (same for all CPUs)
	DWORD Checksum;		//@Field Checksum on header, code and text
} PCDHEADER;

#define PCD_EXTEN		0x01	//@def:(INTERNAL) PCD_EXTEN | Has extension memory
#define PCD_EXTENINIT	0x02	//@def:(INTERNAL) PCD_EXTENINIT | Has ext mem initialization segment

//@Struct PCDSBUSDATA | S-Bus configuration header (see <f PcdRdSbusData> and <f PcdWrSbusData>)
typedef struct tagPCDSBUSDATA
{
	BYTE Station;		//@Field Station number: 0..254, 255=none
	BYTE Baudrate;		//@Field Baud rate: 00=110,01=150,02=300..0A=76800
	BYTE Info[7];		//@Field Bit field array, one BYTE per CPU:
						//@Flag Bit   1,0 | Mode: 00=no S-Bus, 01=master, 10=slave
						//@Flag Bit 4,3,2 | AUX port number: 000..011, 111=none
						//@Flag Bit 7,6,5 | PGU port number: 000..011, 111=none
	BYTE Flags;			//@Field Bit field
						//@Flag Bit 0: | 0=no modem, 1=public line modem
						//@Flag Bit 1: | 0=PCD2 PGU port is RS232, 1=RS485
						//@Flag Bit 2: | Break/Parity, 0x0, 1x1=parity else break
						//@Flag Bit 3: | 0=break/parity, 1=S-Bus data mode
	WORD TsDelay;		//@Field TS Delay in mS: 0..15000 } for private-
	WORD Timeout;		//@Field Timeout in mS:  0..15000 }  line Modems
	WORD TnDelay;		//@Field TN Delay in mS: 0..15000 }
	DWORD Checksum;		//@Field S-Bus data checksum
} PCDSBUSDATA;

typedef PCDSBUSDATA* LPSBUSDATA;

//@Struct PCDEXT | Extension memory header
typedef struct tagPCDEXT
{
	DWORD ExtStart;		//@Field Start line
	DWORD ExtUsed;		//@Field Used in lines
	DWORD ExtFree;		//@Field Free space in lines
} PCDEXT;

//@Struct PCDEXTMAP | Extension memory map
typedef struct tagPCDEXTMAP
{
	PCDEXT Map[PCD_MAXCPU+1];	//@Field Extension memory alloc'n for all CPUs
	DWORD Checksum;				//@Field Checksum on extension memory map
} PCDEXTMAP;

//@Struct PCDEXTHDR | Extended header data structure (not PCD memory image)
//for <f PcdRdExtHeader> and <f PcdWrExtHeader>
//@Comm This structure is obsolete. Cannot be used any longer
//@xref <t enumPcdExtHdrFeatureType>
typedef struct tagPCDEXTHDR
{
	BYTE empty;
} PCDEXTHDR;

//@Struct PCDPROFISBUSDATA | Profi-S-Bus Port Settings
typedef struct tagPCDPROFISBUSDATA
{
    BYTE bEnabled;      //@Field TRUE=it's used
	BYTE Version;		//@Field Version number
    BYTE bPguPort;      //@Field TRUE=full protocol connection, FALSE=reduced protocol
    BYTE bSlave;        //@Field TRUE=slave, FALSE=master
    BYTE SlotNumber;    //@Field 0=B1, 1=B2, 2=onboard
    BYTE Cpu;           //@Field CPU number for multi-cpu system
    BYTE FDLAdds;       //@Field 0..126
    BYTE BaudRate;      //@Field 0=9600 1=19.2K 2=93.75K 3=187.5K 4=500K 5=1.5M 6=3M 7=6M 8=12MBd
    BYTE SAP;           //@Field Service Access Point, default 30 (20..39)
    BYTE Profile;       //@Field PCDPROFISBUSPROFILE_xxx, defined in SaiaConf.h
    BYTE BCSAP;         //@Field Broadcast Service Access Point, default 39 (20..39)
    BYTE Spare1[2];
    WORD SlotTime;      //@Field Slot time
    WORD MaxTsdr;       //@Field Min Tsdr
    WORD MinTsdr;       //@Field Max Tsdr
    BYTE QuietTime;     //@Field Quiet time Tqui
    BYTE SetupTime;     //@Field Setup time Tset
    BYTE Gap;           //@Field Gap
    BYTE Spare2[3];
    BYTE HSA;           //@Field Highest Station Address, default 126
    BYTE Retries;       //@Field Number of retries
    DWORD Ttr;          //@Field Target rotation time
} PCDPROFISBUSDATA;

//@Struct PCDIPMODULEDATA | TCP/IP module configuration
typedef struct tagPCDIPMODULEDATA
{
    BYTE bIPModuleUsed;         //@Field TRUE=IP module fitted
    BYTE Version;               //@Field Version number
    BYTE IPType;                //@Field 0=slave (reduced), 1=S-Bus PGU, 2=RIO mode (no S-Bus adds), FF=no S-Bus (from fw 1.22.04)
    BYTE IPCpu;                 //@Field CPU number for PCD6M3 only
    int IPSlot;                 //@Field Slot number, PCD_SLOTB1|PCD_SLOTB2|PCD_ONBOARD
    int IPNode;                 //@Field IP Node number (0..PCD_MAXIPNODES-1)
    BYTE bIPDHCP;               //@Field use Dynamic Host Configuration Protocol
    char szIPAddress[20];       //@Field IP address, e.g. "10.121.1.44"
    char szIPSubnetMask[20];    //@Field Subnet mask
    char szIPGateway[20];       //@Field Default Gateway address
    // NOTE: currently the same groups are used for both IP modules
    DWORD dwIPServerFlags;      //@Field Flags for each network group server
    DWORD dwIPClientFlags;      //@Field Flags for each network group client
} PCDIPMODULEDATA;


//@Struct PCDEXTHDR2 | Extended header data structure (not PCD memory image)
//for <f PcdRdExtHeader2> and <f PcdWrExtHeader2>
//@xref <t enumPcdExtHdrFeatureType>
typedef struct tagPCDEXTHDR2
{
	// PCD modem strings
	BYTE bHasModem;			//@Field TRUE=modem configured
	char ModemName[36];		//@Field 35 bytes + NUL
	char PCDReset[129];		//@Field 128 bytes + NUL
	char PCDInit[129];		//@Field 128 bytes + NUL

	// Serial Gateway
	BYTE bHasGway;			//@Field TRUE=Gateway configured
	BYTE GwayPort;			//@Field Port number for connection
	BYTE GwayBaud;			//@Field Baud rate: 00=110,01=150,02=300..0A=76800
	BYTE GwayCpu;			//@Field Cpu number
	BYTE GwayMode;			//@Field 0=break, 1=parity, 2=data
	WORD GwayTout;			//@Field Timeout in mS:  0..15000
	WORD GwayTSDel;			//@Field TS Delay in mS: 0..15000 for private-line Modems
	WORD GwayTNDel;			//@Field TN Delay in mS: 0..15000
	WORD Spare;				//@Field Spare, not used
	WORD GwayBreakLen;		//@Field Number of break characters

    BYTE bHasGwayStnRange;  //@Field TRUE=Has serial gateway station range
    BYTE GwayFromStn;       //@Field } Range of S-Bus addresses available
    BYTE GwayToStn;         //@Field } through the serial gateway

	// IP configuration
	BYTE bHasIP;			//@Field TRUE=IP configured
	BYTE IPVersion;			//@Field Version number
	BYTE IPCpu;				//@Field Cpu number for PCD6M3 only
	BYTE IPSlot;			//@Field Slot number, 0=B1, 1=B2, 2=onboard
	WORD IPNode;			//@Field Node number
    BYTE IPType;            //@Field Port type, 0=Slave (reduced), 1=S-Bus PGU (full)
    BYTE bIPDHCP;           //@Field use Dynamic Host Configuration Protocol
	BYTE IPAddress[20];		//@Field IP address of module ex: 10.121.1.44 or 'HostName'
	BYTE IPSubnetMask[20];	//@Field Sub Net mask of module ex: 255.255.255.0 or 'HostName'
	BYTE IPGateway[20];		//@Field Default Gateway address of module ex: 10.121.2.44 or 'HostName'

    PCDIPMODULEDATA IPModule2;      //@Field 2nd IP module (PCD2.M480 only)

    // Profi-S-Bus
    PCDPROFISBUSDATA ProfiSbus1;    //@Field Onboard Profi-S-Bus settings
    PCDPROFISBUSDATA ProfiSbus2;    //@Field Slot B1 Profi-S-Bus settings
    PCDPROFISBUSDATA ProfiSbus3;    //@Field Slot B2 Profi-S-Bus settings

    // S-Bus modem
    BYTE bHasSbusModem;          //@Field TRUE=has s-bus modem
    BYTE ModemPort;              //@Field Port for modem
    BYTE ModemCpu;               //@Field CPU number for multi-cpu system
    BYTE ModemBaudRate;          //@Field PCD_110..PCD_115200
    BYTE ModemSbusMode;          //@Field PCDMODE_DATA/PCDMODE_PARITY
    WORD ModemTsDelay;           //@Field Modem training sequence delay
    WORD ModemTnDelay;           //@Field Modem turnaround time
    WORD ModemTimeout;           //@Field Modem response timeout

    // Profi-S-Bus gateway
    BYTE bHasProfiSbusGway;      //@Field TRUE=has profisbus gateway
    BYTE ProfiSbusGwayCpu;       //@Field CPU number for multi-cpu system
    BYTE ProfiSbusGwaySlot;      //@Field 0=B1, 1=B2, 2=onboard
    BYTE ProfiSbusGwayFromStn;   //@Field } Range of S-Bus addresses available
    BYTE ProfiSbusGwayToStn;     //@Field } through this gateway
    WORD ProfiSbusGwayTimeout;   //@Field Response timeout in ms
    BYTE ProfiSbusGwaySAP;       //@Field Master Service Access Point, default=31 (20..39)
    BYTE ProfiSbusGwaySlaveSAP;  //@Field Slave Service Access Point, default=30 (20..39)
    BYTE ProfiSbusGwayBCSAP;     //@Field Broadcast Service Access Point, def=39 (20..39)

    // TCP/IP gateway
    BYTE bHasTcpipGway;          //@Field TRUE=has tcp/ip gateway
    BYTE TcpipGwayCpu;           //@Field CPU number for multi-cpu system
    BYTE TcpipGwaySlot;          //@Field 0=B1, 1=B2, 2=onboard
    BYTE TcpipGwayFromStn;       //@Field } Range of S-Bus addresses available
    BYTE TcpipGwayToStn;         //@Field } through this gateway
    WORD TcpipGwayTimeout;       //@Field Response timeout in ms

    // Misc
    WORD nUnknownRecords;   //@Field Number of unknown records found
    DWORD dwPcdOptFlags;    //@Field PCD option flags, see PCDOPTFLAG_xxx above

} PCDEXTHDR2;

//@Struct PCDBPITEM | Breakpoint control structure, for <f PcdBreakpoint>
typedef struct tagPCDBPITEM
{
	BYTE  Type;				//@Field Breakpoint type, see #defines
	BYTE  Condition;		//@Field Compare operator '<lt>' '=' '<gt>' '#'
	DWORD Param1;			//@Field Parameter 1, e.g. address of item to check
	DWORD Param2;			//@Field Parameter 2, e.g. value to check for
} PCDBPITEM;

//@Struct PCDBREAKPOINT | Breakpoint
//@xref <f PcdBreakpoint>, <t PCDBPITEM>
//@Comm the two fields are or'ed together
typedef struct tagPCDBREAKPOINT
{
	PCDBPITEM A;			//@Field First  breakpoint <t PCDBPITEM>
	PCDBPITEM B;			//@Field Second breakpoint <t PCDBPITEM>
} PCDBREAKPOINT;
							// Item:         Param1:  Param2:  Cond:
#define PCD_BP_NONE      0	// No breakpoint
#define PCD_BP_STATUS   'S'	// Status        <status> <mask>    no
#define PCD_BP_DISPREG  'D'	// Display reg.  <value>            yes
#define PCD_BP_NSTEPS   'B'	// No. of steps  <nsteps>           no
#define PCD_BP_INDEXREG 'N'	// Index reg.    <value>            yes
#define PCD_BP_INPUT    'I'	// Input state   <adds>    0, 1     no
#define PCD_BP_OUTPUT   'O'	// Output state  <adds>    0, 1     no
#define PCD_BP_FLAG     'F'	// Flag state    <adds>    0, 1     no
#define PCD_BP_IP       'P'	// Instr. ptr.   <value>            yes
#define PCD_BP_REGISTER 'R'	// Register      <adds>   <value>   yes
#define PCD_BP_TIMER    'T'	// Timer         <adds>   <value>   yes
#define PCD_BP_COUNTER  'C'	// Counter       <adds>   <value>   yes
#define PCD_BP_OPCODE   'E'	// Opcode        <opcode>           no
#define PCD_BP_INSTR    'L'	// Instruction   <opcode> <operand> no

#define PCD_BP_GREATER  '>'	// Breakpoint conditions
#define PCD_BP_LESS     '<'
#define PCD_BP_EQUAL    '='
#define PCD_BP_NOTEQUAL '#'

#define PCD_ALLBLOCKS	0	// Step mode: All blocks (default)
#define PCD_ONEBLOCK	1	// Step mode: Current block only

//@Struct PCDHALTREG | Halt register
typedef struct tagPCDHALTREG
{
	char  Text[16+1];	//@Field Failure text, NUL terminated
	DWORD Adds;			//@Field Failure address: 0..999999 = program line number <nl>
						//                  <gt>  999999 = firmware address
    union
    {
        WORD DateTime[6];
        struct
        {
            WORD Year;      //@Field 00..99 (50..99=1950..1999, 00..49=2000..2049)
            WORD Month;     //@Field 1..12
            WORD Day;       //@Field 1..31
            WORD Hour;      //@Field 0..23
            WORD Minute;    //@Field 0..59
            WORD Second;    //@Field 0..59
        };
    };
} PCDHALTREG;

//@Struct PCDHISTORY | History table entry (table is array of up to 32 of these)
typedef struct tagPCDHISTORY
{
	char Message[16+1]; //@Field NUL terminated text describing the error
	DWORD Location;		//@Field Address of error (68000)
	PCDCLOCK Date;		//@Field Date and time of error
} PCDHISTORY;

//@Struct PCDMEMALLOC | Memory allocation, for <f PcdWrAllocation> and <f PcdRdMemoryMap>
typedef struct tagPCDMEMALLOC
{
	DWORD CodeSeg;		//@Field Code segment size in K lines
	DWORD TextSeg;		//@Field Text segment size in K bytes
	DWORD ExtSeg;		//@Field Extension memory segment size in K bytes
} PCDMEMALLOC;

//@Struct PCDCALLSTACK | Block call stack entry, for <f PcdRdCallStack>
typedef struct tagPCDCALLSTACK
{
	BYTE  BlockType;	//@Field 'C'=COB,'X'=XOB,'P'=PB,'F'=FB,'S'=ST,'T'=TR
	WORD  BlockNumber;	//@Field Block's number!
	DWORD ReturnAdds;	//@Field Line number after block call instruction
} PCDCALLSTACK;

//@Struct PCDMULTIBLOCKADDRESS | Structure used by <f PcdChangeBlockEx>
typedef struct tagPCDMULTIBLOCKADDRESS
{
    BYTE    blockType;              //@Field Type of block see <t enumPcdBlockType>
    WORD    blockNumber;            //@Field Block number to change
    DWORD   blockAddress;           //@Field New address of changed block 0 to 0xFFFFFF (3 bytes)
} PCDMULTIBLOCKADDRESS;

/*
@Struct:(INTERNAL) PCDCALLBACK | PCD callback notification structure sent with WM_COPYDATA
 as <e COPYDATASTRUCT.lpData>
*/
typedef struct tagPCDCALLBACK
{
	DWORD	dwSize;			//@Field Size of PCDCALLBACK + Extra data
	DWORD	dwID;   		//@Field Callback ID <t enumComCallback>
	DWORD	dwData;			//@Field Info + ExtraData(optional)
} PCDCALLBACK;

typedef PCDCALLBACK* LPCALLBACK;

//@Struct DNLDBLOCKINFO | Structure used by <f PcdBDnldChgEnd>
typedef struct tagDNLDBLOCKINFO
{
	BYTE	blockType;		//@Field Type of block see <t enumPcdBlockType>
	WORD	blockNumber;	//@Field Block number to change
} DNLDBLOCKINFO;

#pragma pack(pop)      // Normal alignement

/*
@macro | SCOMM_LPNAME | Create a mangled name for a pointer to a function
@parm  | name | Name of the function
@comm the default name is lp_<y functionname><nl>You can also use this macro to call a function
 whithout knowing its mangled name:<nl>s = SCOMM_LPNAME(PcdRdChanSetupFromIni)(NULL, _T("GoOnline"), &PcdOpenData);
@syntax SCOMM_LPNAME(name)
@xref <t SCOMM_LPLOAD>, <t SCOMM_LPDEC>, <t SCOMM_LPDECCP>, <t SCOMM_TDEF>, <t SCOMM_LPTYPE>
@ex To change the name of the pointer variable |
 declare the macro SCOMM_LPNAME(name) before
 including the SCommDll52.h header file.

 #define SCOMM_LPNAME(name) lp_##name
 #include "SCommDll52.h"
*/
#ifndef SCOMM_LPNAME
#define SCOMM_LPNAME(name) lp_##name
#endif

/*
@macro | SCOMM_LPTYPE | Create a mangled type for a pointer to a function
@parm  | name | Name of the function
@comm the default name is LP_<y functionname>
@syntax SCOMM_LPTYPE(name)
@xref <t SCOMM_LPLOAD>, <t SCOMM_LPDEC>, <t SCOMM_LPDECCP>, <t SCOMM_TDEF>, <t SCOMM_LPNAME>
@ex To change the type of the pointer variable |
 declare the macro SCOMM_LPTYPE(name) before
 including the SCommDll52.h header file.

 #define SCOMM_LPTYPE(name) LP_##name
 #include "SCommDll52.h"
*/
#ifndef SCOMM_LPTYPE
#define SCOMM_LPTYPE(name) LP_##name
#endif

/*
@macro | SCOMM_TDEF | Declare the prototype for functions.<nl>Also declares a typedef
 for dynamically loading the function via a pointer.
@parm   | api | API type SCOMMDLL_API or SCOMMUSR_API
@parm   | ret | Type for return value
@parm   | name | Name of the function
@parm   | parm | Parameter declaration
@syntax SCOMM_TDEF(api,  ret, name, parm)
@ex with PcdRdClock |

  SCOMM_TDEF (SCOMMDLL_API, int, PcdRdClock, (PCDCONN PcdConn, PCDCLOCK *lpClock));

  Will generate:

  typedef int (WINAPI *SCOMM_LPTYPE(PcdRdClock)) (PCDCONN PcdConn, PCDCLOCK *lpClock); \
  WINAPI int WINAPI PcdRdClock (PCDCONN PcdConn, PCDCLOCK *lpClock);

  ... by default:
  typedef int (WINAPI *LP_PcdRdClock) (PCDCONN PcdConn, PCDCLOCK *lpClock);
  SCOMMDLL_API int PcdRdClock (PCDCONN PcdConn, PCDCLOCK *lpClock);
@xref <t SCOMM_LPLOAD>, <t SCOMM_LPDEC>, <t SCOMM_LPDECCP>, <t SCOMM_LPTYPE>, <t SCOMM_LPNAME>
*/
#define SCOMM_TDEF(api,  ret, name, parm) \
    typedef ret (WINAPI *SCOMM_LPTYPE(name)) parm; \
    api ret WINAPI name parm

/*
@macro | SCOMM_LPDEC | Define the pointer to the function to load dynamically
 and initialize it to 0.
@comm The name and the type of the pointer can be changed by redefining the macros
 <t SCOMM_LPTYPE> and <t SCOMM_LPNAME><nl><nl>The pointer is initialized and cannot
 be used in classes (C++)
@parm  | name | Name of the function
@syntax SCOMM_LPDEC(name)
@ex with PcdRdClock |

  SCOMM_LPDEC(PcdRdClock);

  Will generate

  SCOMM_LPTYPE(PcdRdClock) SCOMM_LPNAME(PcdRdClock) = (SCOMM_LPTYPE(PcdRdClock))0;

  ... by default:
  LP_PcdRdClock lp_PcdRdClock = (LP_PcdRdClock)0;
@xref <t SCOMM_LPLOAD>, <t SCOMM_TDEF>, <t SCOMM_LPTYPE>, <t SCOMM_LPNAME>
*/
#define SCOMM_LPDEC(name) SCOMM_LPTYPE(name) SCOMM_LPNAME(name) = (SCOMM_LPTYPE(name))0

/*
@macro | SCOMM_LPDECCP | Define the pointer to the function to load dynamically.
@comm The name and the type of the pointer can be changed by redefining the macros
 <t SCOMM_LPTYPE> and <t SCOMM_LPNAME><nl><nl>The pointer is not initialized and can
 be used in classes (C++)
@parm  | name | Name of the function
@syntax SCOMM_LPDECCP(name)
@ex with PcdRdClock |

  SCOMM_LPDECCP(PcdRdClock);

  Will generate

  SCOMM_LPTYPE(PcdRdClock) SCOMM_LPNAME(PcdRdClock);

  ... by default:
  LP_PcdRdClock lp_PcdRdClock;
@xref <t SCOMM_LPLOAD>, <t SCOMM_TDEF>, <t SCOMM_LPTYPE>, <t SCOMM_LPNAME>
*/
#define SCOMM_LPDECCP(name) SCOMM_LPTYPE(name) SCOMM_LPNAME(name)

/*
@macro | SCOMM_LPLOAD | Load the pointer to the function, no error checking is done.
@parm  | handle | Handle of DLL to load the function from
@parm  | name | Name of the function
@comm If the function fails, the variable is set to NULL
@syntax SCOMM_LPLOAD(handle, name)
@ex with PcdRdClock |

  SCOMM_LPLOAD(handle, PcdRdClock);

  Will generate

  SCOMM_LPNAME(PcdRdClock) = (SCOMM_LPTYPE(PcdRdClock))::GetProcAddress(h, "PcdRdClock")

  ... by default:
  lp_PcdRdClock = (LP_PcdRdClock)::GetProcAddress(handle, "PcdRdClock");
@xref <t SCOMM_LPDEC>, <t SCOMM_LPDECCP>, <t SCOMM_TDEF>, <t SCOMM_LPTYPE>, <t SCOMM_LPNAME>
*/
#define SCOMM_LPLOAD(handle, name) SCOMM_LPNAME(name) = (SCOMM_LPTYPE(name))::GetProcAddress(handle, #name)

/*
@type LP_RWPCDBLOCK_NOTIFY | Notification routine for the Write Block telegrams<nl>
 Function prototype is:<nl>
	LRESULT (WINAPI *LP_RWPCDBLOCK_NOTIFY) (LPBYTE lpData, DWORD sizeData, DWORD cookie, DWORD lParam)
@comm This function will be called in 3 different manners:<nl>
1. In <f PcdWRBDataStream> to request data from the client, the application must fill lpData with sizeData bytes of data
   lParam is set to the offset<nl>
   The cookie can be used to pass an arbitary DWORD from the initiating function to this callback<nl>
2. In <f PcdRDBDataStream> to send the data to the application (lpData contains the data and sizeData, the byte count
   lParam is set to the offset<nl>
   The cookie can be used to pass a arbitary DWORD from the initiating function to this callback<nl>
3. This function is also called with lpData = NULL and sizeData = 0 when the PCD is busy (erasing flash), in this case
   lParam is set to PCDBLOCKSTATUSBUSY

If this function returns IDCANCEL (2), the calling function  will send ABORT to the PCD and return a PCD_CANCEL error.<nl>
If this function returns IDABORT (3), the calling function  will send ABORT to the PCD and return a PCD_WRERROR error.<nl>

@xref <f PcdWRBData>, <f PcdRDBData>, <f PcdWRBDataStream>, <f PcdRDBDataStream>, <f PcdBDataDelete>,
 <f PcdBDnld>, <f PcdBDnldChgEnd>, <f PcdWRFBData>, <f PcdRDFBData>, <f PcdRDFBDataStream>, <f PcdFBDataDelete>
*/
typedef LRESULT (WINAPI *LP_RWPCDBLOCK_NOTIFY)(LPBYTE lpData, DWORD sizeData, DWORD cookie, DWORD lParam);

///////////////////////////////////////////////////////////////////////
// Public API Interface

#ifdef __cplusplus
extern "C" {
#endif

// Init/Exit interface
SCOMM_TDEF (SCOMMDLL_API, BOOL, PcdInitInterface, (void));
SCOMM_TDEF (SCOMMDLL_API, void, PcdExitInterface, (void));

// Open/Close/Connect/Lock
SCOMM_TDEF (SCOMMDLL_API, int, PcdComLock, (PCDCONN PcdConn));
SCOMM_TDEF (SCOMMDLL_API, int, PcdComUnlock, (PCDCONN PcdConn));
SCOMM_TDEF (SCOMMDLL_API, int, PcdGetComStatus, (PCDCONN PcdConn));
SCOMM_TDEF (SCOMMDLL_API, LPCSTR, PcdGetComStatusString, (PCDCONN PcdConn));

// Set/Get parameters
SCOMM_TDEF (SCOMMDLL_API, int, PcdSetParentWnd, (PCDCONN PcdConn, HWND hPWDlgParentWnd));

// WComAccu
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdAccu, (PCDCONN PcdConn, LPDWORD lpFlags));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrAccu, (PCDCONN PcdConn, DWORD Flags));

// WComAdds
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdBlockAddress, (PCDCONN PcdConn, DWORD BlockType, DWORD BlockNumber, DWORD Count, LPDWORD lpAddress));

// UComAllo
SCOMM_TDEF (SCOMMUSR_API, int, PcdWrAllocation, (PCDCONN PcdConn, const PCDMEMALLOC* lpMemAlloc, const PCDSBUSDATA* lpSbusData, const PCDEXTHDR2* lpExtHeader, DWORD Control));
SCOMM_TDEF (SCOMMUSR_API, int, PcdRdMemoryMap, (PCDCONN PcdConn, LPDWORD lpMemSize, LPDWORD lpExtSize, PCDMEMALLOC lpMemAlloc[PCD_MAXCPU+1], CHAR lpszProgNames[PCD_MAXCPU+1][9]));
SCOMM_TDEF (SCOMMUSR_API, int, PcdRdMemoryMapEx, (PCDCONN PcdConn, LPDWORD lpMemSize, LPDWORD lpExtSize, PCDMEMALLOC* lpMemAlloc, CHAR* lpszProgNames));

// WComBkpt
SCOMM_TDEF (SCOMMDLL_API, int, PcdBreakpoint, (PCDCONN PcdConn, PCDBREAKPOINT* lpBkpt));
SCOMM_TDEF (SCOMMDLL_API, int, PcdSyncTrace, (PCDCONN PcdConn, DWORD condition, DWORD address, DWORD size));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrSystemBuffer, (PCDCONN PcdConn, DWORD Type, DWORD Address, DWORD Count, LPCDWORD lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdSystemBuffer, (PCDCONN PcdConn, DWORD Type, DWORD Address, DWORD Count, LPDWORD lpData));

// UComBues
SCOMM_TDEF (SCOMMUSR_API, int, PcdBuesExecSql, (PCDCONN PcdConn, LPCSTR lpszSqlCmd, LPSTR* pSqlResult, LPDWORD lpSqlResultSize, LPDWORD lpErrorCode, LPSTR* pError, LPDWORD lpErrorSize));
SCOMM_TDEF (SCOMMUSR_API, int, PcdBuesFreeSql, (LPSTR lpszSqlResult, LPSTR lpszError));
SCOMM_TDEF (SCOMMUSR_API, int, PcdBuesSqlCmd, (PCDCONN PcdConn, DWORD Command));
SCOMM_TDEF (SCOMMUSR_API, int, PcdBuesSpecial, (PCDCONN PcdConn, DWORD Command));
SCOMM_TDEF (SCOMMUSR_API, int, PcdBuesDownload, (PCDCONN PcdConn, DWORD Flags, LPCSTR lpPcdInfo, DWORD PcdInfoVs, LPCSTR lpszPcdFileName, DWORD PcdFileNameVs, LPCSTR lpszFwBuFileName, DWORD FwBuFileVs, LPCSTR lpszFwIsFileName, DWORD FwIsFileNameVs, LPCSTR lpszOutputFileName, LPDWORD ErrorCode, LPSTR lpszError, DWORD ErrorSize, HWND hCallbackWnd));

// WComBues
SCOMM_TDEF (SCOMMDLL_API, int, PcdBuesSendFrame, (PCDCONN PcdConn, DWORD Flags, DWORD SrcCnt, LPCBYTE lpSrc, LPDWORD DstCnt, LPBYTE lpDst));

// WComByte
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdByte, (PCDCONN PcdConn, DWORD Adds, DWORD Count, LPBYTE lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrByte, (PCDCONN PcdConn, DWORD Adds, DWORD Count, LPCBYTE lpData));

// WComCall
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdCallStack, (PCDCONN PcdConn, LPDWORD lpDepth, PCDCALLSTACK* lpCallStack));

// WComCblk
SCOMM_TDEF (SCOMMDLL_API, int, PcdChangeBlock, (PCDCONN PcdConn, DWORD BlockType, DWORD BlockNumber, DWORD Address));
SCOMM_TDEF (SCOMMDLL_API, int, PcdChangeBlockEx, (PCDCONN PcdConn, DWORD command, DWORD timeout, DWORD blockCount, const PCDMULTIBLOCKADDRESS block[], LPDWORD lpResult));

// WComCdlg
SCOMM_TDEF (SCOMMDLL_API, int, PcdGetChannelSetup, (LPCTSTR lpszChannel, LPOPENDATA lpOpenData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdGetChannelSetupEx, (LPCTSTR chanIniFileName, LPCTSTR lpszChannel, LPOPENDATA lpOpenData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdConnectChannel, (PCDCONN* lpPcdConn, LPOPENDATA lpOpenData, DWORD dwFlags, HWND hCallbackWnd));
SCOMM_TDEF (SCOMMDLL_API, int, PcdDisconnectChannel, (PCDCONN PcdConn, DWORD dwFlags, HWND hCallbackWnd));
SCOMM_TDEF (SCOMMDLL_API, int, PcdDialDialog, (HWND hParentWnd, LPOPENDATA lpOpenData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdHangup, (LPOPENDATA lpOpenData, DWORD dwFlags, HWND hParentWnd));

// UComCha
SCOMM_TDEF (SCOMMUSR_API, int, PcdChannelList, (HWND hParentWnd, LPOPENDATA lpOpenData));
SCOMM_TDEF (SCOMMUSR_API, int, PcdConnectionDialog, (HWND hParentWnd, LPOPENDATA lpOpenData));
SCOMM_TDEF (SCOMMUSR_API, int, PcdChannelListEx, (HWND hParentWnd, LPCSTR chanIniFileName, DWORD dwFlags, LPOPENDATA lpOpenData));
SCOMM_TDEF (SCOMMUSR_API, int, PcdConnectionDialogEx, (HWND hParentWnd, LPCSTR chanIniFileName, DWORD dwFlags, LPOPENDATA lpOpenData));
SCOMM_TDEF (SCOMMUSR_API, int, PcdConnectionDialogEx2, (HWND hParentWnd, LPCSTR persIniFileName, LPCSTR chanIniFileName, DWORD dwFlags, LPOPENDATA lpOpenData));


// WComChan
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdChanSetupFromIni, (LPCSTR lpszIniFile, LPCSTR lpszAppName, LPOPENDATA lpOpenData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrChanSetupToIni, (LPCSTR lpszIniFile, LPCSTR lpszAppName, LPCOPENDATA lpOpenData));

// WComChanEx
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdChanSetupFromIniEx, (LPCSTR lpszChanIniFile, LPCSTR lpszIniFile, LPCSTR lpszAppName, LPOPENDATA lpOpenData));

// WComodat
SCOMM_TDEF (SCOMMDLL_API, int, PcdReadOpendata, (LPOPENDATA lpOpenData, LPCSTR lpFileName, LPCSTR lpSectionName, DWORD Flags));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWriteOpendata, (LPCOPENDATA lpOpenData, LPCSTR lpFileName, LPCSTR lpSectionName, DWORD Flags));

// JPE 24.01.02 New channel access functions
SCOMM_TDEF (SCOMMDLL_API, int, PcdGetChannelList, (LPCTSTR lpszIniFile, LPBYTE lpChannelList, DWORD maxSize));
SCOMM_TDEF (SCOMMDLL_API, int, PcdGetChannel, (LPCTSTR lpszIniFile, LPCTSTR lpszChannelName, LPOPENDATA lpOpenData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdSetChannel, (LPCTSTR lpszIniFile, LPCTSTR lpszChannelName, LPOPENDATA lpOpenData));

// WComClok
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdClock, (PCDCONN PcdConn, PCDCLOCK* lpClock));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrClock, (PCDCONN PcdConn, PCDCLOCK* lpClock));

// WComClr
SCOMM_TDEF (SCOMMDLL_API, int, PcdClearText, (PCDCONN PcdConn, DWORD Number));
SCOMM_TDEF (SCOMMDLL_API, int, PcdDelete, (PCDCONN PcdConn, DWORD Cpu, DWORD Control));
SCOMM_TDEF (SCOMMDLL_API, int, PcdClear, (PCDCONN PcdConn, DWORD Type));

// UComComp
SCOMM_TDEF (SCOMMUSR_API, int, PcdCompare, (PCDCONN PcdConn, DWORD Cpu, LPCSTR lpszFileName));

// WComConn
SCOMM_TDEF (SCOMMDLL_API, int, PcdConnect, (PCDCONN PcdConn, DWORD Cpu, DWORD Station, BOOL bGetPassword));
SCOMM_TDEF (SCOMMDLL_API, int, PcdGetOpenData, (PCDCONN PcdConn, LPOPENDATA lpOpenData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdGetProtocolName, (LPCOPENDATA lpcOpenData, LPSTR lpName, DWORD MaxNameSize));
SCOMM_TDEF (SCOMMDLL_API, int, PcdSetStation, (PCDCONN PcdConn, DWORD Station));
SCOMM_TDEF (SCOMMDLL_API, int, PcdGetPortName, (LPCOPENDATA lpcOpenData, LPSTR lpszName, DWORD MaxNameSize));
SCOMM_TDEF (SCOMMDLL_API, int, PcdGetBaudrateName, (LPCOPENDATA lpcOpenData, LPSTR lpszName, DWORD MaxNameSize));

SCOMM_TDEF (SCOMMDLL_API, int, PcdSetStationEx, (PCDCONN PcdConn, DWORD Station, LPCSTR ExtraStation));
SCOMM_TDEF (SCOMMDLL_API, int, PcdConnectEx, (PCDCONN PcdConn, DWORD Cpu, DWORD Station, LPCSTR ExtraStation, BOOL bGetPassword));

// WComConv
SCOMM_TDEF (SCOMMDLL_API, DWORD, PcdIEEEToFFP, (float f));
SCOMM_TDEF (SCOMMDLL_API, float, PcdFFPToIEEE, (DWORD ffp));
SCOMM_TDEF (SCOMMDLL_API, DWORD, PcdAToFFP, (LPCSTR lpAscii, LPINT lpStatus));
SCOMM_TDEF (SCOMMDLL_API, LPSTR, PcdFFPToA, (DWORD ffp, LPSTR lpAscii));
SCOMM_TDEF (SCOMMDLL_API, WORD, PcdGetFwFeatures, (LPCSTR lpszVersion));

// WComCtrl
SCOMM_TDEF (SCOMMDLL_API, int, PcdRun, (PCDCONN PcdConn, DWORD Cpu));
SCOMM_TDEF (SCOMMDLL_API, int, PcdStop, (PCDCONN PcdConn, DWORD Cpu));
SCOMM_TDEF (SCOMMDLL_API, int, PcdSingleStep, (PCDCONN PcdConn));
SCOMM_TDEF (SCOMMDLL_API, int, PcdCheckRun, (PCDCONN PcdConn, LPBOOL lpCpuRun));

// WComCurb
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdCurrentBlock, (PCDCONN PcdConn, LPDWORD lpBlockType, LPDWORD lpBlockNumber));

// WComDblk
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdDB, (PCDCONN PcdConn, DWORD Number, LPDWORD lpItems,LPDWORD lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdDBItem, (PCDCONN PcdConn, DWORD Number, DWORD Item, LPDWORD lpItems, LPDWORD lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrDB, (PCDCONN PcdConn, DWORD Number, DWORD Size, LPCDWORD lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrDBItem, (PCDCONN PcdConn, DWORD Number, DWORD Item, DWORD nItems, LPCDWORD lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdMakeDB, (PCDCONN PcdConn, DWORD Number, DWORD Size));
SCOMM_TDEF (SCOMMDLL_API, int, PcdClearDB, (PCDCONN PcdConn, DWORD Number));
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdDBBytes, (PCDCONN PcdConn, DWORD Number, DWORD Offset, DWORD nBytes, LPBYTE lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrDBBytes, (PCDCONN PcdConn, DWORD Number, DWORD Offset, DWORD nBytes, LPCBYTE lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdDBX, (PCDCONN PcdConn, DWORD Number, DWORD Offset, DWORD nBytes, LPBYTE lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdGetDiagnostics, (PCDCONN PcdConn, DWORD Code, DWORD Count, LPCBYTE lpParams, LPDWORD lpRresult));

// WComDisp
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdDispReg, (PCDCONN PcdConn, LPDWORD lpDispReg));

// UComDnbk
SCOMM_TDEF (SCOMMUSR_API, int, PcdDownloadBlock, (PCDCONN PcdConn, LPCSTR lpszFileName, DWORD BlockType, DWORD BlockNumber, DWORD Control, HWND hCallbackWnd));

// UComDnld
SCOMM_TDEF (SCOMMUSR_API, int, PcdDownload, (PCDCONN PcdConn, LPCSTR lpszFileName, DWORD Cpu, DWORD Control, HWND hCallbackWnd));

// WComEepr
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdEEPROM, (PCDCONN PcdConn, DWORD Address, DWORD Count, LPDWORD lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrEEPROM, (PCDCONN PcdConn, DWORD Address, DWORD Data));

// WComExec
SCOMM_TDEF (SCOMMDLL_API, int, PcdExecute, (PCDCONN PcdConn, LPCDWORD lpInst, DWORD Length));
SCOMM_TDEF (SCOMMDLL_API, int, PcdExecuteXob, (PCDCONN PcdConn, DWORD Xob));

// WComFlas
SCOMM_TDEF (SCOMMDLL_API, int, PcdFlashErase, (PCDCONN PcdConn, DWORD EraseType));

// UComFlash
SCOMM_TDEF (SCOMMUSR_API, int, PcdCopyToFlash, (PCDCONN PcdConn, DWORD dwFlags, HWND hCallbackWnd));
SCOMM_TDEF (SCOMMUSR_API, int, PcdCopyToFlashEx, (PCDCONN PcdConn, DWORD command, DWORD options, LPCSTR directory, LPCSTR file, LPBYTE lpDataDst, LPDWORD lpcDataDst, DWORD dwFlags, HWND hCallbackWnd));

// WComFwup
SCOMM_TDEF (SCOMMDLL_API, int, PcdFwDownload, (PCDCONN PcdConn, DWORD Code, DWORD Count, LPCBYTE lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdFwDownloadData, (PCDCONN PcdConn, DWORD Address, DWORD Count, LPCBYTE lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdFlashcard, (PCDCONN PcdConn, DWORD Type, LPBYTE lpDataSrc, DWORD cDataSrc, LPBYTE lpDataDst, LPDWORD lpcDataDst));

// WComHalt
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdHalt, (PCDCONN PcdConn, PCDHALTREG* lpHalt));
SCOMM_TDEF (SCOMMDLL_API, int, Pcd68000ToUser, (PCDCONN PcdConn, DWORD Address, LPSTR lpszAdds));

// WComHist
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdHistory, (PCDCONN PcdConn, PCDHISTORY* lpHist, LPDWORD lpEntries, LPDWORD lpLast));

// WComIdat
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdSystemInfo, (PCDCONN PcdConn,    DWORD Type, LPBYTE lpDataSrc, DWORD cDataSrc, LPBYTE lpDataDst, LPDWORD lpcDataDst));
SCOMM_TDEF (SCOMMDLL_API, int, PcdGetInternalData, (PCDCONN PcdConn, LPINTERNALDATA lpInternalData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdCheckInternalData, (PCDCONN PcdConn, DWORD nLevel, LPINTERNALDATA lpInternalData));

// WComIdel
SCOMM_TDEF (SCOMMDLL_API, int, PcdInsertLine, (PCDCONN PcdConn, DWORD Address));
SCOMM_TDEF (SCOMMDLL_API, int, PcdDeleteLine, (PCDCONN PcdConn, DWORD Address));

// WComIndx
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdIndex, (PCDCONN PcdConn, LPDWORD lpIndex));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrIndex, (PCDCONN PcdConn, DWORD Index));

// WComIof
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdIOF, (PCDCONN PcdConn, DWORD Type, DWORD Address, DWORD Count, LPSTR lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrOF, (PCDCONN PcdConn,    DWORD Type, DWORD Address, DWORD Count, LPCSTR lpData));

// WComIptr
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdIP, (PCDCONN PcdConn, LPDWORD lpIP));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrIP, (PCDCONN PcdConn, DWORD IP));

// WComLan
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdLAN, (PCDCONN PcdConn, LPSTR lpszInfo));

// WComMem
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdMemory, (PCDCONN PcdConn, DWORD Address, DWORD Count, LPDWORD lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrMemory, (PCDCONN PcdConn, DWORD Address, DWORD Count, LPCDWORD lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdHeader, (PCDCONN PcdConn, DWORD Cpu, PCDHEADER* lpHeader));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrHeader, (PCDCONN PcdConn, DWORD Cpu, const PCDHEADER* lpHeader));
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdExtMap, (PCDCONN PcdConn, PCDEXTMAP* lpExtMap));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrExtMap, (PCDCONN PcdConn, const PCDEXTMAP* lpExtMap));
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdSbusData, (PCDCONN PcdConn, PCDSBUSDATA* lpSbusData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrSbusData, (PCDCONN PcdConn, const PCDSBUSDATA* lpSbusData));

// WComModi
SCOMM_TDEF (SCOMMDLL_API, int, PcdModified, (PCDCONN PcdConn, DWORD Cpu));

// WComMsgs
SCOMM_TDEF (SCOMMDLL_API, LPCSTR, PcdMessage, (DWORD Status));

// UComOdlg
SCOMM_TDEF (SCOMMUSR_API, HWND, PcdStatusDialog, (HWND hParentWnd, LPOPENDATA lpOpenData));
SCOMM_TDEF (SCOMMUSR_API, int, PcdCloseStatusWindow, (HWND hStatusWnd));
SCOMM_TDEF (SCOMMUSR_API, int, PcdMessageBox, (LPCTSTR lpszTitle, LPCTSTR lpszText, WORD nType));
SCOMM_TDEF (SCOMMUSR_API, int, PcdPhonebookDialog, (HWND hParentWnd, DWORD Flags, LPPHONEBOOK lpPhonebook));

// WComOpen
SCOMM_TDEF (SCOMMDLL_API, int, PcdComOpen, (PCDCONN* lpPcdConn, LPOPENDATA lpOpenData, DWORD wFlags, HWND hCallbackWnd));
SCOMM_TDEF (SCOMMDLL_API, int, PcdComClose, (PCDCONN PcdConn, DWORD wFlags, HWND hCallbackWnd));

SCOMM_TDEF (SCOMMDLL_API, int, PcdDllVersion, (void));
SCOMM_TDEF (SCOMMDLL_API, int, PcdComUnloadDrv, (BOOL bUnloadOnLastClose));
SCOMM_TDEF (SCOMMDLL_API, int, PcdDrvVersion, (void));

// WComP800
SCOMM_TDEF (SCOMMDLL_API, int, PcdTestP800, (PCDCONN PcdConn));
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdP8Version, (PCDCONN PcdConn, LPSTR lpszVersion));

// WComParm
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdParameter, (PCDCONN PcdConn, DWORD ParmNum, LPSTR lpszParm, BOOL bInit));

// WComPass
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdP1Password, (PCDCONN PcdConn, LPSTR lpszPassword, LPDWORD lpAccess));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrP1Password, (PCDCONN PcdConn, LPCSTR Password, DWORD Access));

// WComPoll
SCOMM_TDEF (SCOMMDLL_API, int, PcdPoll, (PCDCONN PcdConn));

// WComProg
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdProgram, (PCDCONN PcdConn, DWORD LineNo, DWORD nLines, LPDWORD lpProg));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrProgram, (PCDCONN PcdConn, DWORD LineNo, DWORD nLines, LPCDWORD lpProg));

// WComPwrd
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrPassword, (PCDCONN PcdConn, LPCSTR lpszPassword, DWORD Timeout));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrPasswordDw, (PCDCONN PcdConn, DWORD dwPassword, DWORD Timeout));
SCOMM_TDEF (SCOMMDLL_API, int, PcdEnterPassword, (PCDCONN PcdConn, LPCSTR lpszPassword));
SCOMM_TDEF (SCOMMDLL_API, int, PcdGetPassword, (PCDCONN PcdConn, HWND hPWDlgParentWnd));

// WComRact
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdActiveTRs, (PCDCONN PcdConn, DWORD nSB, WORD ActiveTRs[32], LPDWORD lpLength));
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdActiveTRsEx, (PCDCONN PcdConn, DWORD nSB, WORD* ActiveTRs, LPDWORD lpLength));

// WComRdfw
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdFWVersions, (PCDCONN PcdConn, CHAR pFWVers[PCD_MAXCPU+1][3+1]));
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdFWVersionsEx, (PCDCONN PcdConn, CHAR* pFWVers));

// WComRest
SCOMM_TDEF (SCOMMDLL_API, int, PcdRestart, (PCDCONN PcdConn, DWORD Cpu, DWORD Type));
SCOMM_TDEF (SCOMMDLL_API, int, PcdSetNoClearOutputs, (PCDCONN PcdConn, BOOL bNoClearOutputs));

// WComRstn
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdStation, (PCDCONN PcdConn, LPDWORD lpStation));

// WComRtc
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdRTC, (PCDCONN PcdConn, DWORD Type, DWORD Address, DWORD Count, LPDWORD lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrRTC, (PCDCONN PcdConn, DWORD Type, DWORD Address, DWORD Count, LPCDWORD lpData));

// WComSbus
SCOMM_TDEF (SCOMMDLL_API, int, PcdGetComStatistics, (PCDCONN PcdConn, LPCOMSTATISTICS lpComStatistics, BOOL bClear));
SCOMM_TDEF (SCOMMDLL_API, int, PcdRawXmit, (PCDCONN PcdConn, DWORD SrcCnt, LPCBYTE lpSrc, LPDWORD DstCnt, LPBYTE lpDst));

// WComSel
SCOMM_TDEF (SCOMMDLL_API, int, PcdSelectCpu, (PCDCONN PcdConn, DWORD Cpu));

// UComSetc
SCOMM_TDEF (SCOMMUSR_API, int, PcdSetClockDialog, (PCDCONN PcdConn, HWND hParentWnd));

// WComSize
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdBlockSize, (PCDCONN PcdConn, DWORD BlockType, DWORD BlockNumber, DWORD Count, LPDWORD lpSize));

// WComSmod
SCOMM_TDEF (SCOMMDLL_API, int, PcdSetStepMode, (PCDCONN PcdConn, DWORD StepMode));

// WComStat
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdStatus, (PCDCONN PcdConn, DWORD Cpu, LPDWORD lpStatus));

// WComText
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdTextType, (PCDCONN PcdConn, DWORD Number, LPDWORD lpType));
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdText, (PCDCONN PcdConn, DWORD Number, LPDWORD lpLength, LPSTR lpszText));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrText, (PCDCONN PcdConn, DWORD Number, DWORD Size, LPCSTR lpszText));
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdTextChar, (PCDCONN PcdConn, DWORD Number, DWORD Offset, LPDWORD lpLength, LPSTR lpszText));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrTextChar, (PCDCONN PcdConn, DWORD Number, DWORD Offset, DWORD Length, LPCSTR lpszText));
SCOMM_TDEF (SCOMMDLL_API, int, PcdMakeText, (PCDCONN PcdConn, DWORD Number, DWORD Size));

// UComUpld
SCOMM_TDEF (SCOMMUSR_API, int, PcdUpload, (PCDCONN PcdConn, DWORD Cpu, LPCSTR lpszFilename, DWORD Control, HWND hCallbackWnd));

// WComvers
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdVersion, (PCDCONN PcdConn, LPSTR lpszVersion));
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdConnectedCpu, (PCDCONN PcdConn, LPDWORD lpCpu, LPDWORD lpCpuType));

// WComwstn
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrStation, (PCDCONN PcdConn, DWORD Station));

// WComWeb
SCOMM_TDEF (SCOMMDLL_API, int, PcdWebServerSComm, (PCDCONN PcdConn, DWORD RidNumber, DWORD PackN, LPCBYTE lpSrcData, DWORD SrcSize, LPDWORD lpAidNumber, LPBYTE lpDestData, LPDWORD lpDestSize));

// WComXhdr
SCOMM_TDEF (SCOMMDLL_API, int, PcdExtHdrOk, (PCDCONN PcdConn, LPBOOL lpExtHdrOk));
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdExtHeader, (PCDCONN PcdConn, PCDEXTHDR* lpExtHdr));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrExtHeader, (PCDCONN PcdConn, const PCDEXTHDR* lpExtHdr));
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdExtHeader2, (PCDCONN PcdConn, PCDEXTHDR2* lpExtHdr));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrExtHeader2, (PCDCONN PcdConn, const PCDEXTHDR2* lpExtHdr));

// WComRWB
SCOMM_TDEF (SCOMMDLL_API, int, PcdRWBlock, (PCDCONN PcdConn, DWORD FunType, DWORD SrcCnt, LPCBYTE lpSrc, LPDWORD lpDstCnt, LPBYTE lpDst));

// UComRWB
SCOMM_TDEF (SCOMMUSR_API, int, PcdWRBData, (PCDCONN PcdConn, DWORD blockType, DWORD blockNumber, DWORD blockAddress, DWORD dataSize, LPCBYTE lpData, DWORD cookie, LP_RWPCDBLOCK_NOTIFY notify));
SCOMM_TDEF (SCOMMUSR_API, int, PcdRDBData, (PCDCONN PcdConn, DWORD blockType, DWORD blockNumber, DWORD blockAddress, LPDWORD lpDataSize, LPBYTE lpData, DWORD cookie, LP_RWPCDBLOCK_NOTIFY notify));
SCOMM_TDEF (SCOMMUSR_API, int, PcdWRBDataStream, (PCDCONN PcdConn, DWORD blockType, DWORD blockNumber, DWORD blockSize, LPCBYTE lpData, DWORD cookie, LP_RWPCDBLOCK_NOTIFY notify));
SCOMM_TDEF (SCOMMUSR_API, int, PcdRDBDataStream, (PCDCONN PcdConn, DWORD blockType, DWORD blockNumber, LPDWORD dataSize, LPBYTE lpData, DWORD cookie, LP_RWPCDBLOCK_NOTIFY notify));
SCOMM_TDEF (SCOMMUSR_API, int, PcdBDataSize, (PCDCONN PcdConn, DWORD blockType, DWORD blockNumber, LPDWORD lpSize, LPDWORD lpCrc32));
SCOMM_TDEF (SCOMMUSR_API, int, PcdBDataDelete, (PCDCONN PcdConn, DWORD blockType, DWORD blockNumber, DWORD cookie, LP_RWPCDBLOCK_NOTIFY notify));
SCOMM_TDEF (SCOMMUSR_API, int, PcdBDnld, (PCDCONN PcdConn, DWORD command, DWORD option, LPDWORD status, DWORD cookie, LP_RWPCDBLOCK_NOTIFY notify));
SCOMM_TDEF (SCOMMUSR_API, int, PcdBDnldChgEnd, (PCDCONN PcdConn, DWORD count, const DNLDBLOCKINFO* lpBlockInfo, LPDWORD status, DWORD cookie, LP_RWPCDBLOCK_NOTIFY notify));

// UComRWFB
SCOMM_TDEF (SCOMMUSR_API, int, PcdWRFBData, (PCDCONN PcdConn, DWORD blockType, LPCSTR blockName, DWORD blockAddress, DWORD dataSize, LPCBYTE lpData, DWORD cookie, LP_RWPCDBLOCK_NOTIFY notify));
SCOMM_TDEF (SCOMMUSR_API, int, PcdRDFBData, (PCDCONN PcdConn, DWORD blockType, LPCSTR blockName, DWORD blockAddress, LPDWORD lpDataSize, LPBYTE lpData, DWORD cookie, LP_RWPCDBLOCK_NOTIFY notify));
SCOMM_TDEF (SCOMMUSR_API, int, PcdWRFBDataStream, (PCDCONN PcdConn, DWORD blockType, LPCSTR blockName, DWORD blockSize, LPCBYTE lpData, DWORD cookie, LP_RWPCDBLOCK_NOTIFY notify));
SCOMM_TDEF (SCOMMUSR_API, int, PcdRDFBDataStream, (PCDCONN PcdConn, DWORD blockType, LPCSTR blockName, LPDWORD dataSize, LPBYTE lpData, DWORD cookie, LP_RWPCDBLOCK_NOTIFY notify));
SCOMM_TDEF (SCOMMUSR_API, int, PcdFBDataSize, (PCDCONN PcdConn, DWORD blockType, LPCSTR blockName, LPDWORD lpSize, LPDWORD lpCrc32));
SCOMM_TDEF (SCOMMUSR_API, int, PcdFBDataDelete, (PCDCONN PcdConn, DWORD blockType, LPCSTR blockName, DWORD cookie, LP_RWPCDBLOCK_NOTIFY notify));

// WComMM
SCOMM_TDEF (SCOMMDLL_API, int, PcdMultiMedia, (PCDCONN PcdConn, LPCSTR lpcRequest, LPDWORD ResponseMaxSize, LPSTR lpResponse));

// WComTrf
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdTempReg, (PCDCONN PcdConn, DWORD Address, DWORD Count, LPDWORD lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrTempReg, (PCDCONN PcdConn, DWORD Address, DWORD Count, LPCDWORD lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdTempFlag, (PCDCONN PcdConn, DWORD Address, DWORD Count, LPSTR lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrTempFlag, (PCDCONN PcdConn, DWORD Address, DWORD Count, LPCSTR lpData));

// WComTsk
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdTaskData, (PCDCONN PcdConn, DWORD Type, DWORD TaskId, DWORD Address, DWORD Count, LPBYTE lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrTaskData, (PCDCONN PcdConn, DWORD Type, DWORD TaskId, DWORD Address, DWORD Count, LPCBYTE lpData));
SCOMM_TDEF (SCOMMDLL_API, int, PcdRdPeriData, (PCDCONN PcdConn, DWORD Type, DWORD VAddress, DWORD Count, LPBYTE lpData, LPDWORD lpStatus));
SCOMM_TDEF (SCOMMDLL_API, int, PcdWrPeriData, (PCDCONN PcdConn, DWORD Type, DWORD VAddress, DWORD Count, LPCBYTE lpData, LPDWORD lpStatus));

#ifdef __cplusplus
}
#endif

