/************************************************************************
*                                                                       *
*       PROGRAM     SASM52.DLL - SAIA PCD ASSEMBLER DLL 32-BIT          *
*       MODULE      SASM52.H                                            *
*       AUTHOR      Matt Harvey                                         *
*                                                                       *
*************************************************************************

REVISION HISTORY
28-Sep-06   Initial edit, from Sasm32.h.
...
10-Nov-08   Increased PCDSYM.Flags and PCDSYM2.Flags from BYTE to DWORD.
            PcdSym2Str() returns string lengths increeased!
01-Dec-08   Add PCDSYMFLG_REFERENCED to indicate if a symbol is referenced.
            Add PCDSCOPEFLAG_REFERENCED for filtering only the referenced
            symbols.
20-Jan-2011 Matt: Version for Unicode apps which may call this ANSI DLL
            LPCxxx  -> const char* 
            LPxxx   -> char*
            TCHAR   -> char
            TBYTE   -> unsigned char
            CString -> CStringA
            _T      -> removed

DESCRPITION
Include file for users of Sasm52.dll
*/

#ifndef __SASM52_H
#define __SASM52_H

#ifdef __SASM52_DLL
#define SASM_API __declspec(dllexport)
#else
#define SASM_API __declspec(dllimport)
#endif

#ifndef _INC_WINDOWS
#include <windows.h>
#endif

#pragma pack(push, 1)           // 1-byte packing

#if defined(__cplusplus)
extern "C" {
#endif

typedef int* HPCDFILE;          // PCD file handle
typedef int* HPCDRESLIST;       // Resource list handle
typedef HGLOBAL HPCDSYMTAB;     // Symbol table handle

// DBX Numbers

#define DBX_FMS                 0
#define DBX_DP_ONBOARD          1
#define DBX_DP_SLOT_B1          2
#define DBX_LON                 3
#define DBX_DP_SLOT_B2          4
#define DBX_IPMODULE            5
#define DBX_REGISTRY            6   // Program Information in INI format
#define DBX_CSF                 7
#define DBX_WEB_SERVER          8
#define DBX_PROGRAM_INFO        9
#define DBX_FIRST_INIT          10  // First-time Initialization Data
#define DBX_START_INIT          11  // Start-up Initialization Data
#define DBX_CAN                 12
#define DBX_IOCONFIG            13
#define DBX_IOCONFIG2           14
#define DBX_CANOPEN             15
#define DBX_RIOMASTERCONFIG     16  // S-Bus RIO master configuration
#define DBX_RIOCONFIG           17  // Ethernet RIO station configuration
#define DBX_RIOMANAGERCONFIG    18  // Ethernet RIO manager configuration
#define DBX_ELINECONFIG         19  // E-Line device configuration, media base addresses etc

#define DBX_SEG_FIRST           20  // 20 DBXs for $DBXSEG..$ENDDBXSEG directives
#define DBX_SEG_COUNT           20
#define DBX_SEG_LAST            (DBX_SEG_FIRST+DBX_SEG_COUNT-1)

#define DBX_PLUGIN_FIRST        100 // 20 DBXs for plugins
#define DBX_PLUGIN_LAST         119

// IB Numbers

#define IB_SQL                  0           // BuES SQL data
#define IB_LON                  3           // LON configuration
#define IB_REDUNDANCY           1           // CPU redundancy data definition
//#define IB_FBOXINITINFO         5           // Fbox initialization code information, for Fupla download in run
#define IB_STATIC_DATA          5           // Static data information block for download in run
#define IB_FIRST_INIT           10          // First-time Initialization Data
#define IB_START_INIT           11          // Start-up Initialization Data

// Macros for converting register addresses
// register addresses are special format:
// 10aa aaaa aaaa aabb
// where:
// 10 = media code (MCR)
// aa aaaa aaaa aa = least significant 12 bits of address
// bb = most significant 2 bits (register block number)

#define REGNUMTOOPA(regNum) (((regNum & 0x0FFF) << 2) + ((regNum >> 12) & 3))
#define OPATOREGNUM(opa)    (((opa & 3) << 12) + ((opa >> 2) & 0x0FFF))

// Flags for PcdMake()

#define SASMFLG_NEVERCLOSE      0x0002
#define SASMFLG_ALWAYSCLOSE     0x0004
#define SASMFLG_USEHWND         0x0008
#define SASMFLG_KEEPLINKFILE    0x0010
#define SASMFLG_MAKEALL         0x0020
#define SASMFLG_QUITON1STERROR  0x0040
#define SASMFLG_NOMAP           0x0080
#define SASMFLG_NOSYMBOLS       0x0100
#define SASMFLG_CALLSBS         0x0200
#define SASMFLG_NOLINK          0x0400
#define SASMFLG_SPM             0x0800
#define SASMFLG_SYNTAXCHECK     0x1000
#define SASMFLG_SILENT          0x2000
#define SASMFLG_BUILDLIB        0x4000
#define SASMFLG_NOUSERCHECK     0x8000

// Assembler/linker functions

SASM_API int WINAPI PcdMake(const char* lpszMakeFileName, WORD Flags,
                            UINT FAR *lpErrors, UINT FAR *lpWarnings);
SASM_API int WINAPI PcdMakeEx(HWND hWnd, const char* lpszMakeFileName, WORD Flags,
                              UINT FAR *lpErrors, UINT FAR *lpWarnings);
SASM_API void WINAPI PcdCancelMake(void);
SASM_API char* WINAPI PcdGetMakeErrorMessage(int ErrNo, char* lpszMessage);
SASM_API BOOL WINAPI PcdIsMaking(void);
SASM_API int WINAPI PcdIsInternalSymbolValid(const char* pszSymName, BOOL bAcceptGroups);
SASM_API int WINAPI PcdGetMakeVersion(void);
#define PcdSasmGetVersion PcdGetMakeVersion
SASM_API BOOL WINAPI PcdCreateSeditSyntaxFiles(void);
SASM_API BOOL WINAPI PcdIsEncryptedFile(const char* pszFileName);
SASM_API BOOL WINAPI PcdGetAbsSym(char* pszSymbol, int type, int value);
// NOTE: int return value, not BOOL: 0=valid, >0 = invalid, see below...
SASM_API int WINAPI PcdIsSymbolValid(const char* pszSymName, BOOL bAcceptGroups);

SASM_API char* WINAPI PcdIEEEFloatToA(DWORD dwFloat, char* pszFloat);
SASM_API char* WINAPI PcdIEEEDoubleToA(DWORD dwDouble[2], char* pszDouble);
SASM_API BOOL WINAPI PcdAToIEEEDouble(const char* pszDouble, DWORD pdwDouble[2]);
SASM_API BOOL WINAPI PcdAToIEEEFloat(const char* pszFloat, DWORD* pdwFloat);

// Return values for PcdIsSymbolValid()
#define SASM_OK                     0   // was PCD_OK
#define PCD_FIRST_CHAR_INVALID      1
#define PCD_INVALID_CHARACTERS      2
#define PCD_RESERVED_WORD           3
#define PCD_SYMBOL_TOO_LONG         4
#define PCD_SYMBOL_TOO_SHORT        5

// Error return values
#define SASM_OK                     0
#define SASM_HADERRORS              1
#define SASM_CANCELLED              2
#define SASM_BADPARAMETER           3
#define SASM_INPROGRESS             4
#define SASM_BADFILENAME            5
#define SASM_CHDIRFAILED            6
#define SASM_FILEOPENERROR          7
#define SASM_FILEREADERROR          8
#define SASM_FILEWRITEERROR         9
#define SASM_FILECREATEERROR        10
#define SASM_ALLOCFAILED            11
#define SASM_TOOMANYINCPATHS        12
#define SASM_TOOMANYINCFILES        13
#define SASM_NOPCDFILENAME          14
#define SASM_NOSOURCEFILENAME       15
#define SASM_INTERNALERROR          16
#define SASM_FATALASMERROR          17
#define SASM_FATALLINKERROR         18
#define SASM_BADFILE                19
#define SASM_NOTFOUND               20
#define SASM_WRONGFILE              21
#define SASM_LASTERROR              21

// Extension memory record header (Text/DB 4000..7999)
// NOTE: All 2 and 4-byte values are in Motorola format, MS byte first

typedef struct tagPCDEXTSEGRECHDR
{
    WORD Number;            // Text/DM number, Motorola format
    BYTE Type;              // 0x08=text, 0x20=DB
    BYTE Flags;             // not used
    WORD Length;            // In bytes for text, in 32-bit elements for DB
    WORD Spare;             // not used
                            // Variable length data follows
} PCDEXTSEGRECHDR;

// DBX record header
// NOTE: All 2 and 4-byte values are in Motorola format, MS byte first

typedef struct tagPCDDBXRECHDR
{
    WORD Number;            // DBX number, Motorola format, bit 15 always set
    BYTE Type;              // not used
    BYTE Flags;             // not used
    DWORD Length;           // Length in bytes, Motorola format
                            // Variable length data follows
} PCDDBXRECHDR;

// IB (Information Block) record header
// NOTE: All 2 and 4-byte values are in Motorola format, MS byte first

typedef struct tagPCDIBRECHDR
{
    WORD Number;            // IB number, Motorola format, bit 15 always set
    BYTE Type;              // 0x02 = IB
    BYTE Flags;             // not used
    DWORD Length;           // Length in bytes, Motorola format
                            // Variable length data follows
} PCDIBRECHDR;


//////////////////////////////////////////////////////////////////////
// Symbol table functions (were in SSYM32.H)

#define PCD_MAXSYMBOLLEN    80  // Max. chars in a symbol } EXCLUDES NUL
#define PCD_MAXCOMMENTLEN   80  // Max. chars in comment  } TERMINATOR!!
#define PCD_MAXMODNAMELEN   64  // Max module name length, including .SRC } ''

// Function return values

#define PCDSYM_OK           0   // Success
#define PCDSYM_CANTOPEN    -1   // Can't open PCD file
#define PCDSYM_BADFILE     -2   // Invalid PCD file
#define PCDSYM_RDERROR     -3   // File read error (disk I/O error)
#define PCDSYM_NOSYMBOLS   -4   // No symbol data in .PCD file
#define PCDSYM_NOTFOUND    -5   // Symbol not found or end of symbols
#define PCDSYM_BADSTHCS    -6   // Bad checksum on Symbol Table Header
#define PCDSYM_BADMLRCS    -7   // Bad checksum on Module List Record
#define PCDSYM_BADAASCS    -8   // Bad checksum on Auto Allocation Segment
#define PCDSYM_BADSYMCS    -9   // Bad checksum on public/local symbol segment
#define PCDSYM_OUTOFMEM    -10  // Out of memory, memory alloc failed
#define PCDSYM_BADPARM     -11  // Bad hPcdFile handle or invalid parameter
#define PCDSYM_BADPARAM    -11  // Better spelling!
#define PCDSYM_BADPCDVERS  -12  // Old PCD file version
#define PCDSYM_BADEXTCS    -13  // Bad checksum on OBJ file external segment
#define PCDSYM_LASTERROR   -13

// Data types, for "Type" in PCDSYM and for PcdIsResourceUsed() "Type" value
// Those marked * are also offsets to $AUTO records (PCDAUTOSEGREC) in PCDAUTOSEG

#define PCD_NOTYPE      0       // Untyped 32-bit signed constant
#define PCD_LABTYPE     1       // Label (cannot be public)
#define PCD_IOTYPE      2       // Input or Output (I|O)
#define PCD_FTYPE       3       // Flag
#define PCD_TCTYPE      4       // Timer or Counter (T|C)
#define PCD_RTYPE       5       // Register
#define PCD_KTYPE       6       // K constant (12 bits, 0..16383)
#define PCD_FBTYPE      7       // Function Block
#define PCD_PBTYPE      8       // Program Block
#define PCD_COBTYPE     9       // Cyclic Organisation Block
#define PCD_XOBTYPE     10      // Exception Organisation Block
#define PCD_SBTYPE      11      // Sequential Block
#define PCD_STTYPE      12      // Step or initial step
#define PCD_TRTYPE      13      // Transition
#define PCD_TEXTTYPE    14      // Text
#define PCD_SEMATYPE    15      // Semaphore
#define PCD_FBPARAMTYPE 16      // FB parameter (=)
#define PCD_DBTYPE      17      // Data Block
#define PCD_TTYPE       18      // Timer
#define PCD_CTYPE       19      // Counter
#define PCD_ITYPE       20      // Input
#define PCD_OTYPE       21      // Output
#define PCD_GROUPTYPE   22      // Group name from $GROUP   NOTE: do not use, group types are no longer in the symbol table
#define PCD_DBXTYPE     23      // DBX
#define PCD_IBTYPE      24      // Information Block IB
#define PCD_TIMETYPE    25      // Duration, T#nnS|MS
#define PCD_MACROTYPE   26      // Macro name
#define PCD_LASTTYPE    26      // Last type number

// Additional data types for PcdFileReadBlock() and PcdFileGetBlockSize()

#define PCD_CODESEG     100     // Code segment
#define PCD_TEXTSEG     101     // Text/DB segment (0..3999)
#define PCD_EXTSEG      102     // Extension memory text/DB seg. (4000..7999)
#define PCD_IBSEG       103     // Information Block segment
#define PCD_FILESEG     104     // Downloadable file

// Offsets to $AUTO records (PCDAUTOSEGREC) in PCDAUTOSEG

#define PCD_VOLFTYPE     0      // Volatile flag
#define PCD_RAMTEXTTYPE 1       // Extension memory text
#define PCD_RAMDBTYPE   2       // Extension memory DB

// Units types, for "Units" in PCDSYM

#define PCD_DECUNITS    0       // Decimal
#define PCD_BINUNITS    1       // Binary
#define PCD_ASCUNITS    2       // ASCII
#define PCD_FFPUNITS    3       // Motorola fast floating point (FFP)
#define PCD_HEXUNITS    4       // Hexadecimal
#define PCD_SECUNITS    5       // Duration in secs, T#nnS } for PCD_TIMETYPE
#define PCD_MSUNITS     6       // Duration in ms, T#nnMS  } only
#define PCD_IEEEUNITS   7       // IEEE Float
#define PCD_UNITSMASK   0x07

// Direction, for FB and SF parameters

#define PARAM_IN        1
#define PARAM_OUT       2
#define PARAM_INOUT     3

// Generic or mixed types, others use PCD_xTYPE, for FB and SF parameters

#define PARAM_BOOL      200     // 1-bit constant (0 or 1)
#define PARAM_INT       201     // 16-bit signed constant
#define PARAM_UINT      202     // 16-bit unsigned constant
#define PARAM_RFLOAT    203     // R FLOAT
#define PARAM_R_K       204     // R|K
#define PARAM_DB_X      205     // DB|X
#define PARAM_DB_X_R    206     // DB|X|R
#define PARAM_RIEEE     207     // R IEEE
#define PARAM_RTC       208     // R|T|C
#define PARAM_IOF       209     // I|O|F
#define PARAM_X_K       210     // X|K
#define PARAM_DB_K      211     // DB|K
#define PARAM_DB_X_K    212     // DB|X|K
#define PARAM_X_R       213     // X|R
#define PARAM_DB_R      214     // DB|R
#define PARAM_X_R_K     215     // X|R|K
#define PARAM_DB_R_K    216     // DB|R|K
#define PARAM_DB_X_R_K  217     // DB|X|R|K
#define PARAM_F_K       218     // F|K
#define PARAM_ANY       219     // any type
#define PARAM_R_F       220     // R|F
#define PARAM_X_R_F     221     // X|R|F
#define PARAM_LAST      221     // last value

// Symbol Table Header

typedef struct tagPCDSYMTABHDR
{
    DWORD ModuleListOffset;     // Offset to module list in file
    DWORD NumModules;           // Number of modules linked
    DWORD AutoSegOffset;        // Offset to Auto Allocation Segment in file
    DWORD PublicSymbolOffset;   // Offset to Public Symbol Segment in file
    DWORD NumPublicSymbols;     // Number of public symbols
    BYTE Flags;                 // Flags, 0x0002=has comments
    BYTE RevCode;               // PCD file revision 'a'..'z'

} PCDSYMTABHDR;

// Module List Record
// Array record, one for each SRC module linked, see PCDSYMTABHDR.NumModules

typedef struct tagPCDMODLISTREC
{
    char ModuleName[PCD_MAXMODNAMELEN+1]; // Source file name and extension (no path)
    char ObjName[_MAX_PATH];    // Full object file name
    char DateTime[18];          // Assembly date/time in ASCII
    DWORD CodeStartLine;        // Code start line
    DWORD CodeLines;            // Size of code in lines
    DWORD TextBytes;            // Size of text in bytes
    DWORD ExtBytes;             // Size of extension memory in bytes
    DWORD LocalSymbolOffset;    // Offset to Local Symbol Segment in file
    DWORD NumLocalSymbols;      // Number of local symbols
    WORD Flags;                 // PCD_FLGxxx, see below

} PCDMODLISTREC;

// Values for "Flags" in PCDSYMTABHDR and PCDMODLISTREC

#define PCD_FLGHASXOB16     0x0001  // Module contains user-defined XOB 16

// Auto Allocation Segment Record

typedef struct tagPCDAUTOSEGREC
{
    BYTE Type;          // PCD_RTYPE, PCD_TTYPE etc, 0=AUTO not supported
    BYTE Spare;         // Unused at present
    WORD Start;         // Start element
    WORD End;           // End element of AUTO range
    WORD Used;          // No. of elements actually used

} PCDAUTOSEGREC;

typedef PCDAUTOSEGREC PCDAUTOSEG[22];   // Auto Allocation Segment

// Public or Local Symbol

typedef struct tagPCDSYM
{
    char* lpszSymbolName;   // Symbol name
    char* lpszComment;      // Symbol's comment
    BYTE Module;            // No. of module where Public symbol defined
    BYTE Type;              // Media type, PCD_RTYPE etc.
    BYTE Units;             // Units, PCD_DECUNITS etc.
    BYTE Spare;             // padding
    DWORD Flags;            // PCDSYMFLG_xxx, see below
    LONG Value;             // 32-bit value
    UINT ArraySize;         // No. of elements in array
    DWORD BlockID;          // Block signature for blocks and block-local symbols
    char* lpszExpression;   // Symbol's expression, NULL if none

} PCDSYM;

// Flags for PCDSYM.Flags
//>>>these are also used for Flags BYTE in PCDPUBSYMREC/PCDLOCSYMREC/OBJPUBSYMREC/OBJLOCSYMREC
//see #define LUNITSMASK 0x0F
#define PCDSYMFLG_ISARRAY       0x00000010  // Symbol is an array, see ArraySize
#define PCDSYMFLG_VOLRAM        0x00000020  // VOL F or RAM TEXT/DB
#define PCDSYMFLG_AUTOALLOC     0x00000040  // Auto-allocated symbol, else absolute
#define PCDSYMFLG_TAGGROUPS     0x00000080  // Has tag group names list
//<<<
#define PCDSYMFLG_PUBLIC        0x00000001  // Symbol is PUBLic 
#define PCDSYMFLG_IST           0x00000002  // Initial Step (for PCD_STTYPE only)
#define PCDSYMFLG_FLOAT         0x00000004  // R FLOAT
#define PCDSYMFLG_GLOBALINC     0x00000008  // Symbol is in a global $include file
#define PCDSYMFLG_IEEE          0x00000100  // R IEEE
#define PCDSYMFLG_REFERENCED    0x00000200  // Symbol has been referenced (used)
#define PCDSYMFLG_DEFINED       0x00000400  // Symbol declared with DEF (local)
#define PCDSYMFLG_S_ADJUST      0x00000800  // symbol is nonvolatile fupla online adjust parameter
//#define PCDSYMFLG_FBOXDATA      0x00001000  // symbol is defined in an FBox, disable DIR if its address changes
#define PCDSYMFLG_FUPLA         0x00002000  // block is coded in Fupla (block type symbols only)
#define PCDSYMFLG_SYMEDITOR     0x00004000  // symbol is defined in symbol editor (.sy5 $HIDE .sym)

// Old version of PCDSYM
// Uses character arrays and has no ArraySize or Flags

typedef struct tagPCDSYMVB
{
    char szSymbolName[PCD_MAXSYMBOLLEN+1];  // Symbol name
    char szComment[PCD_MAXCOMMENTLEN+1];    // Symbol's comment
    BYTE Module;            // No. of module where Public symbol defined
    BYTE Type;              // Media type, PCD_RTYPE etc.
    BYTE Units;             // Units, PCD_DECUNITS etc.
    LONG Value;             // 32-bit value

} PCDSYMVB;

// Data from $CPU, $STATION and $PCDVER directives

#define PCDMAXVERRECS 50    // Max. number of different $PCDVER directives

typedef struct tagPCDVERREC // Data from $PCDVER directive
{
    char szType[8];         // PCD main type, e.g. "D4M??5" or "NTFW"
    char szVersion[4];      // Firmware version, e.g. "004": if szType is "NTFW" then this contains a DWORD min version
    char szIntVersion[4];   // Internal version, e.g. "3A": if szType is "NTFW" then this contains a DWORD max version, 0=no max

} PCDVERREC;

typedef struct tagPCDTYPESEG
{
    int Cpu;                // CPU number, from $CPU
    int Station;            // S-BUS station number, from $STATION
    int nVerRecs;           // Number of VerRecs
    PCDVERREC VerRec[PCDMAXVERRECS];    // $PCDVER directive data

} PCDTYPESEG;

// Flags stored in PcdIndex[0], read with PcdFileGetIndex(..., 0);
// These new flags were put into an unused index slot to save changing
// the format of the PCD file and then having versioning problems
// in all old format PCD files the value is always zero
#define PCDFLGX_WRITESTOTEXTSEG  0x00000001     // text segment written to
#define PCDFLGX_HAS_UID_DB       0x00000002     // DB __PCD_UID__ is present, DB number in PcdIndex[1]
#define PCDFLGX_HAS_BUILDKEYDB   0x00000004     // DB __PCD_CURRENT_BUILD_KEY__, DB number in PcdIndex[2]
#define PCDFLGX_EXTENDEDFLAGS    0x00000008     // program uses Flags 8192..14335
#define PCDFLGX_RF64K            0x00000010     // program uses R/F addresses in 64K address format (media type in opcode)

// PCD file functions

SASM_API HPCDFILE WINAPI PcdFileOpen(const char* pszPcdFilename, int* pErrNo);
SASM_API void WINAPI PcdFileClose(HPCDFILE hPcdFile);
SASM_API DWORD WINAPI PcdFileGetIndex(HPCDFILE hPcdFile, int offset);
SASM_API int WINAPI PcdFileGetHeader(HPCDFILE hPcdFile, void* /*PCDHEADER**/ pPcdHeader);
SASM_API int WINAPI PcdFileGetBlockSize(HPCDFILE hPcdFile, int BlockType,
                                        int BlockNumber, int* pBlockSize);
SASM_API int WINAPI PcdFileReadBlock(HPCDFILE hPcdFile, int BlockType,
                                     int BlockNumber, int Offset,
                                     int BytesToRead, BYTE* pDest);
SASM_API int WINAPI PcdFileGetNextBlock(HPCDFILE hPcdFile, int Segment,
                                        int* pBlockType, int* pBlockNumber,
                                        int* pBlockSize, BOOL bFirst);
SASM_API int WINAPI PcdFileGetNextBlockEx(HPCDFILE hPcdFile, int Segment,
                                          int* pBlockType, int* pBlockNumber,
                                          int* pBlockSize, int* pBlockOffset, BOOL bFirst);
SASM_API int WINAPI PcdFileGetNextCalledBlock(HPCDFILE hPcdFile,
                                              int BlockType, int BlockNumber,
                                              PCDSYM* pSym, BOOL bNoSymbolInfo,
                                              BOOL bFirst);
SASM_API int WINAPI PcdFileGetBlockInfo(HPCDFILE hPcdFile,
                                        int BlockType, int BlockNumber,
                                        int* pAddress, int* pSize);
SASM_API int WINAPI PcdFileGetBlockInfoEx(HPCDFILE hPcdFile, 
                                        int BlockType, int BlockNumber, 
                                        int* pAddress, int* pSize, int* pOffset);

// Symbol table functions

SASM_API int WINAPI PcdGetSymTabHeader(const char*, PCDSYMTABHDR *);
SASM_API int WINAPI PcdGetModListRec(const char*, int, PCDMODLISTREC *);
SASM_API int WINAPI PcdGetAutoSeg(const char* lpszPcdFilename, PCDAUTOSEG *lpAutoSeg);
// Note: These have been superseded by the PcdDataListXXX() functions below
SASM_API HPCDSYMTAB WINAPI PcdPublicSymOpen(const char*, LPLONG, LPINT);
SASM_API HPCDSYMTAB WINAPI PcdLocalSymOpen(const char*, const char*, LPLONG, LPINT);
SASM_API int WINAPI PcdSymGetNext(HPCDSYMTAB, PCDSYM *);
SASM_API int WINAPI PcdSymFindNext(HPCDSYMTAB, const char*, PCDSYM *);
SASM_API int WINAPI PcdSymGet(HPCDSYMTAB, const char*, PCDSYM *);
SASM_API int WINAPI PcdSymGetNextVB(HPCDSYMTAB, PCDSYMVB *);
SASM_API int WINAPI PcdSymFindNextVB(HPCDSYMTAB, const char*, PCDSYMVB *);
SASM_API int WINAPI PcdSymGetVB(HPCDSYMTAB, const char*, PCDSYMVB *);
SASM_API int WINAPI PcdSymRewind(HPCDSYMTAB);
SASM_API int WINAPI PcdSymClose(HPCDSYMTAB);
SASM_API int WINAPI PcdGetTypeSeg(const char*, PCDTYPESEG *);
SASM_API int WINAPI PcdSymVersion(void);
SASM_API void WINAPI PcdSymToStr(char* pszType, char* pszValue, PCDSYM* pSym);
SASM_API void WINAPI PcdGetSymbolFromFileName(LPCTSTR pszFName, LPTSTR pszSymbol);


// Resource list functions

SASM_API HPCDRESLIST WINAPI PcdResListOpen(const char* szPcdFilename, LPINT lpErrNo);
SASM_API int WINAPI PcdResListClose(HPCDRESLIST hResList);
SASM_API BOOL WINAPI PcdResListIsUsed(HPCDRESLIST hResList, int Type, int Number);
SASM_API int WINAPI PcdResListGetNext(HPCDRESLIST hResList, int Type, int Number,
                                      PCDSYM* pSymData, BOOL bFirst);
SASM_API BOOL WINAPI PcdResListExtendedSFC(HPCDRESLIST hResList);
SASM_API BOOL WINAPI PcdResListExtendedRegs(HPCDRESLIST hResList);
SASM_API BOOL WINAPI PcdResListMediaUsed(HPCDRESLIST hResList, int type, int startAdds, int endAdds);
SASM_API BOOL WINAPI PcdMediaValidForPCD3T770(HPCDRESLIST hResList);
SASM_API BOOL WINAPI PcdResListExtendedFlags(HPCDRESLIST hResList);
SASM_API BOOL WINAPI PcdResListRF64K(HPCDRESLIST hResList);


//////////////////////////////////////////////////////////////////////////////
// New data list functions

// String types

typedef char PCDSYMBOLNAME [PCD_MAXSYMBOLLEN+1];
typedef char PCDMODULENAME [_MAX_PATH];
typedef char PCDCOMMENT    [PCD_MAXCOMMENTLEN+1];
typedef char PCDFILENAME   [_MAX_PATH]; // (_MAX_PATH includes the NUL)
typedef char PCDERRORMSG   [256];

// Type flags for PCDDATAFILTER.typeFlags, OR together

typedef enum
{
    PCDTYPEFLAG_R       = 0x00000001,   // registers
    PCDTYPEFLAG_T       = 0x00000002,   // timers
    PCDTYPEFLAG_C       = 0x00000004,   // counters
    PCDTYPEFLAG_F       = 0x00000008,   // flags
    PCDTYPEFLAG_I       = 0x00000010,   // inputs
    PCDTYPEFLAG_O       = 0x00000020,   // outputs
    PCDTYPEFLAG_X       = 0x00000040,   // texts
    PCDTYPEFLAG_DB      = 0x00000080,   // data blocks
    PCDTYPEFLAG_DBX     = 0x00000100,   // extended data blocks
    PCDTYPEFLAG_IB      = 0x00000200,   // information blocks
    PCDTYPEFLAG_S       = 0x00000400,   // semaphores
    PCDTYPEFLAG_MEDIA   = 0x00000FFF,   // all the above media

    PCDTYPEFLAG_COB     = 0x00010000,   // COBs
    PCDTYPEFLAG_XOB     = 0x00020000,   // XOBs
    PCDTYPEFLAG_PB      = 0x00040000,   // PBs
    PCDTYPEFLAG_FB      = 0x00080000,   // FBs
    PCDTYPEFLAG_SB      = 0x00100000,   // SBs
    PCDTYPEFLAG_ST      = 0x00200000,   // STs
    PCDTYPEFLAG_TR      = 0x00400000,   // TRs
    PCDTYPEFLAG_BLOCKS  = 0x00FF0000,   // all the above blocks

    PCDTYPEFLAG_FBPARAM = 0x08000000,   // FB parameters (=)
    PCDTYPEFLAG_K       = 0x10000000,   // constants
    PCDTYPEFLAG_TIME    = 0x20000000,   // time type
    PCDTYPEFLAG_MACRO   = 0x40000000,   // macros
    PCDTYPEFLAG_GROUP   = 0x80000000,   // groups

    PCDTYPEFLAG_SYMBOL  = 0x01000000,   // for PcdBrowseForSymbolDlg() only
    PCDTYPEFLAG_ADDRESS = 0x02000000    // for PcdBrowseForSymbolDlg() only

} PCDTYPEFLAGS;

// Scope flags for PCDDATAFILTER.scopeFlags, OR together

typedef enum
{
    PCDSCOPEFLAG_FBOX       = 0x00000001,   // fbox internal
    PCDSCOPEFLAG_MACRO      = 0x00000002,   // macro internal
    PCDSCOPEFLAG_LOCAL      = 0x00000010,   // module local
    PCDSCOPEFLAG_PUBLIC     = 0x00000020,   // publics
    PCDSCOPEFLAG_SYSTEM     = 0x00000040,   // system
    PCDSCOPEFLAG_ABS        = 0x00000080,   // absolute addresses
    PCDSCOPEFLAG_REFERENCED = 0x00000004,   // referenced symbols only

    // old: compare symbol name string to what (both low=compare to symbol)
    PCDSCOPEFLAG_COMM       = 0x00000100,   // compares symbolName to comment only
    PCDSCOPEFLAG_SYMCOMM    = 0x00000200,   // compares symbolName to symbol name AND comment

    // new: from v2.3, define which field to compare symbolName, none=compare to symbol
    PCDSCOPEFLAG_CMPSYMBOL  = 0x00001000,   // compare symbolName to symbol (default)
    PCDSCOPEFLAG_CMPCOMMENT = 0x00002000,   // compare symbolName to comment
    PCDSCOPEFLAG_CMPTAGS    = 0x00004000,   // compare symbolName to tag names

} PCDSCOPEFLAGS;

// Data filter for sorting, see PcdDataListSort()

typedef struct tagPCDDATAFILTER
{
    PCDSYMBOLNAME symbolName;   // name or partial symbol name (with * or ?)
    PCDMODULENAME moduleName;   // module to search in (""=all modules)
    PCDTYPEFLAGS typeFlags;     // PCDTYPEFLAG_xxx, which types to return, 0=all
    PCDSCOPEFLAGS scopeFlags;   // PCDSCOPEFLAG_xxx, which scopes to return, 0=default
    BOOL byValue;               // TRUE=search for specific value
    DWORD value;                // specific value to search for

} PCDDATAFILTER;

// Symbol, same values as in old PCDSYM structure

typedef struct tagPCDSYM2
{
    PCDSYMBOLNAME SymbolName;   // Symbol name
    PCDCOMMENT Comment;         // Symbol's comment
    BYTE Module;                // No. of module where symbol is defined
    BYTE Type;                  // Media type, PCD_RTYPE etc.
    BYTE Units;                 // Units, PCD_DECUNITS etc.
    BYTE Spare;                 // padding
    DWORD Flags;                // PCDSYMFLG_xxx
    DWORD Value;                // 32-bit value
    WORD ArraySize;             // No. of elements in array
    DWORD XrefOffset;           // Offset to xref list in OBJ file
    DWORD Index;                // Index in sorted symbol list
    WORD nTagGroupNames;        // Number of tag group names
    WORD nAttributes;           // Number of attributes
    DWORD BlockID;              // Block signature for blocks and block-local symbols
    char Expression[80];        // Symbol's expression

} PCDSYM2;

// 'Sort by' selectors (selects primary sort)

typedef enum
{                               // 1       2       3       4
    PCDSORTBY_TYPE      = 0,    // type    value   symbol  module
    PCDSORTBY_VALUE     = 1,    // value   type    symbol  module
    PCDSORTBY_SYMBOL    = 2,    // symbol  type    value   module
    PCDSORTBY_MODULE    = 3,    // module  symbol  type    value
    PCDSORTBY_SCOPE     = 4     // scope   symbol  type    value
    //PCDSORTBY_TAG : use PcdDataListSortByTag()

} PCDSORTBY;

// 'Search by' selectors

typedef enum
{
    PCDSEARCHBY_SYMBOL    = 0,   // symbol name
    PCDSEARCHBY_TYPEVALUE = 1    // type and value

} PCDSEARCHBY;


// Cross-reference list record, OBJ file revision 212 and above

typedef struct tagPCDXREFREC
{
    // Bit 0 = XRFFLG_DEFINED   symbol is defined on this line
    // Bit 1 = XRFFLG_WRITTEN   written
    // Bit 2 = XRFFLG_INDEXED   indexed
    // Bit 3 = XRFFLG_FUPLA     Fupla reference
    // Bit 4 = XRFFLG_GRAFTEC   Graftec reference
    // Bit 5 = XRFFLG_BLKDEF    block definition, COB/XOB/PB/FB/SB/ST/TR
    BYTE Flags;
    // File number containing the reference
    // 0=main file, 1..255=include file 0..254
    // value is offset into IncludeFileList array
    BYTE FileNo;
    // Line number in source module
    WORD LineNo;
    // For Fupla Fbox
    WORD FupPage;           // page number
    WORD FupFboxPos;        // postion on page: (row<<8)+col
    // Fupla or Graftec block
    BYTE BlockType;         // block type PCD_COBTYPE etc
    //BYTE BlockNameNo;     // offset into array of block names
    WORD BlockNameNo;       // '', changed to WORD from revcode 217
    // For macro
    WORD MacroLine;         // line number in macro, 0=not in a macro
    WORD MacroNameNo;       // offset into array of macro names

} PCDXREFREC;

// Cross-reference list data

typedef struct tagPCDXREFDATA
{
    PCDXREFREC XrefRec;
    char szSymbolName[PCD_MAXSYMBOLLEN+1];
    char szMacroName[PCD_MAXSYMBOLLEN+1];
    char szBlockName[PCD_MAXSYMBOLLEN+1];
    char szFileName[_MAX_PATH + 1];
    char szInfo[_MAX_PATH + 1];

} PCDXREFDATA;

// Version with CPU path, for WM_COPYDATA to SPM
typedef struct tagPCDXREFDATAEX
{
    PCDXREFDATA xrefData;
    char szCpuPath[_MAX_PATH + 1];
    HWND hWndParent;

} PCDXREFDATAEX;

typedef void* HPCDDATALIST;     // the handle is really a pointer

SASM_API HPCDDATALIST WINAPI PcdDataListOpen(const char* pcdFilename, int* pErrorNo);
SASM_API void WINAPI PcdDataListClose(HPCDDATALIST hList);
SASM_API int WINAPI PcdDataListBuild(HPCDDATALIST hList, PCDDATAFILTER* pFilter,
                                     PCDSORTBY sortBy);
SASM_API int WINAPI PcdDataListSort(HPCDDATALIST hList, PCDSORTBY sortBy);
SASM_API int WINAPI PcdDataListGet(HPCDDATALIST hList, PCDSYM2* pSym, int index);
SASM_API int WINAPI PcdDataListFind(HPCDDATALIST hList, PCDSYM2* pSym,
                                    PCDSEARCHBY searchBy, int startIndex,
                                    BOOL searchBackwards);
SASM_API void WINAPI PcdDataListGetErrorMsg(HPCDDATALIST hList, PCDERRORMSG* errorMsg,
                                           int errorNo);
SASM_API void WINAPI PcdSym2ToStr(HPCDDATALIST hList, PCDSYM2* pSym, char* pszType,
                                  char* pszValue, char* pszScope, char* pszSubtype,
                                  const char** pszModuleName);
SASM_API const char* WINAPI PcdDataListGetModuleName(HPCDDATALIST hList, int moduleNumber);
SASM_API int WINAPI PcdDataListGetModuleNameEx(HPCDDATALIST hList, int moduleNumber, 
                                               char* pszModuleName);
SASM_API int WINAPI PcdDataListGetActualFileName(HPCDDATALIST hList, PCDSYM2* pSym, 
                                                 char* pszFileName);
SASM_API int WINAPI PcdDataListSortByTag(HPCDDATALIST hList, const char* pszTag);
SASM_API DWORD* WINAPI PcdDataListGetIndex(HPCDDATALIST hList);

// Cross-reference dialog box
SASM_API int WINAPI PcdCreateXrefDlg(HWND hWndParent, const char* pcdFilename, PCDSYM2* pSym, 
                                     const char* pszModuleName);
SASM_API void WINAPI PcdDestroyXrefDlg(void);

// 'Is symbol used' functions
typedef void* HPCDSYMUSEDTABLE;
/*old
SASM_API HPCDSYMUSEDTABLE WINAPI PcdSymUsedTableCreate(const char* pszCpuPath, const char* pszModulePath);
SASM_API void WINAPI PcdSymUsedTableDestroy(HPCDSYMUSEDTABLE hTable);
SASM_API int WINAPI PcdIsSymUsed(HPCDSYMUSEDTABLE hTable, const char* pszSymbolName, BOOL bOneSymbol);
*/
SASM_API HANDLE WINAPI PcdSymUsedTableCreate(const char* pszCpuPath, const char* pszFilePath, int* pErrorNo);
SASM_API void WINAPI PcdSymUsedTableDestroy(HANDLE hTable);
SASM_API int WINAPI PcdIsSymUsed(HANDLE hTable, const char* pszSymbol, BOOL bOneSymbol, BOOL bGlobal);

SASM_API BOOL WINAPI PcdIsOperandWritten(const char* pszMnemonic, int nOperand);


// Documentation generator function
// Data for PcdCreateDocumentation
typedef struct tagPCDDOCDATA
{
    void* pCpuFiles;            // actually CFilesArray* 
    const char* pszCpuDir;
    const char* pszCpuName;
    const char* pszPcdPath;
    const char* pszDlsPath;
    DWORD dwOptionsFlags;       // DOCFLG_xxxx
    int nLinesPerPage;
    HWND hWndParent;

} PCDDOCDATA;

// Flags for dwOptionsFlags
#define DOCFLG_SRCLINENUMBERS   0x0001
#define DOCFLG_LOCALSBYTYPE     0x0002
#define DOCFLG_LOCALSBYNAME     0x0004
#define DOCFLG_GLOBALSBYTYPE    0x0008
#define DOCFLG_GLOBALSBYNAME    0x0010
#define DOCFLG_AUTOCOMMENTS     0x0020
#define DOCFLG_MACRONAMES       0x0040
#define DOCFLG_SILENT           0x0080      // no error message dialog box or beeps
#define DOCFLG_NOGRAPHICS       0x0100
#define DOCFLG_NOPAGEBREAKS     0x0200      // no titles or page breaks

// Note: void* pStrErrMsg is actually CStringA*
SASM_API BOOL WINAPI PcdCreateDocumentation(PCDDOCDATA* pPcdDocData, void* pStrErrMsg);

// Global Symbol Picker dialog box, in SymDlg.dll
BOOL WINAPI PcdBrowseForSymbolDlg(HWND hParentWnd, const char* pszCpuPath, DWORD dwFlags, 
UINT helpMessage, char* pszSymbol, char* pszType, char* pszValue, char* pszComment);


//////////////////////////////////////////////////////////////////////////////
// New block information/changed blocks functions

// only allowed if Scomm32.h has been included first
#if defined(__SCOMM32_H) || defined(SCOMMDLL_API)

// Unique program identifier
typedef INT64 PCDUID;

// Changed block list record
typedef struct tagPCDCHANGEBLOCKREC
{
    WORD number;        // block number
    BYTE type;          // block type
    BYTE status;        // PCD_CB_xxx
    // these will be 0 if the the block was deleted:
    DWORD newSize;      // size of new block in bytes
    DWORD newOffset;    // byte offset from start of segment in PCD of new block
    // these will be 0 of the block has been added:
    DWORD oldSize;      // size of old block in bytes
    DWORD oldOffset;    // byte offset from start of segment in PCD of old block

} PCDCHANGEDBLOCKREC;

typedef void* HPCDCHANGEDBLOCKLIST; // changed block list handle

// Changed block status values
#define PCD_CB_DONTKNOW     (-1)
#define PCD_CB_UNCHANGED    0
#define PCD_CB_CHANGED      1
#define PCD_CB_BIGGER       2
#define PCD_CB_SMALLER      3
#define PCD_CB_ADDED        4
#define PCD_CB_DELETED      5
#define PCD_CB_TYPECHANGED  6

SASM_API int WINAPI PcdBlockInfoBuildCreate(const char* pcdFilename, const char* infoPath);
SASM_API void WINAPI PcdBlockInfoDelete(const char* infoPath, int segmentFlags);
SASM_API int WINAPI PcdBlockInfoPcdCreate(const char* infoPath, PCDHEADER* pPcdHeader,
    PCDEXT* pExtHeader, int segmentFlags, PCDUID uid);
SASM_API int WINAPI PcdBlockInfoGetLastError(char* errMsg, int errNo);

//SASM_API BOOL WINAPI PcdCreateProjectUID(PCDHEADER* pHeader, PCDUID* pUid);
//SASM_API BOOL WINAPI PcdGetProjectUid(const char* infoPath, PCDUID* pUid);

SASM_API int WINAPI PcdDisplayMemFreeFiles(HWND hWndParent, const char* infoPath);
SASM_API int WINAPI PcdDisplayChangedBlocks(HWND hWndParent, const char* pcdFilename, const char* infoPath);

SASM_API HPCDCHANGEDBLOCKLIST WINAPI PcdChangedBlocksListOpen(const char* pcdFilename,
    const char* infoPath, int* pnChangedBlocks, int* pErrNo);
SASM_API void WINAPI PcdChangedBlocksListClose(HPCDCHANGEDBLOCKLIST hList);
SASM_API PCDUID WINAPI PcdChangedBlocksListGetUid(HPCDCHANGEDBLOCKLIST hList);
SASM_API PCDCHANGEDBLOCKREC* WINAPI PcdChangedBlocksListGetRec(HPCDCHANGEDBLOCKLIST hList, int recNo);
SASM_API BOOL WINAPI PcdChangedBlocksListGetRecEx(HPCDCHANGEDBLOCKLIST hList, int recNo,
    PCDCHANGEDBLOCKREC* pChangedBlockRec);
SASM_API BOOL WINAPI PcdBlockInfoBuildExists(const char* infoPath);
SASM_API int WINAPI PcdFileGetUid(const char* pszFileName, PCDUID* pUid, int* pDbNumber);
//SASM_API int WINAPI PcdFileGetBuildKeyDBNumber(const char* pszFileName);

// Block download list record
typedef struct tagPCDBLOCKDOWNLOADLISTREC
{
    BYTE segment;       // PCD_CODESEG etc
    BYTE status;        // PCD_CB_xxx
    BYTE flags;         // not used yet, always 0
    BYTE type;          // PCD_COBTYPE etc
    WORD number;
    // these will be 0 if the the block was deleted:
    DWORD newSize;      // size of new block in bytes
    DWORD newOffset;    // byte offset from start of segment in PCD of new block
    // these will be 0 of the block has been added:
    DWORD oldSize;      // size of old block in bytes
    DWORD oldOffset;    // byte offset from start of segment in PCD of old block

} PCDBLOCKDOWNLOADLISTREC;

typedef void* HPCDBLOCKDOWNLOADLIST;
SASM_API HPCDBLOCKDOWNLOADLIST WINAPI PcdChangedBlocksDownloadListOpen
    (const char* pcdFilename, const char* infoPath, int* nBlocks, BOOL bRunning);
SASM_API int WINAPI PcdChangedBlocksDownloadListClose
    (HPCDBLOCKDOWNLOADLIST hDnldList, BOOL bSave);
SASM_API PCDBLOCKDOWNLOADLISTREC* WINAPI PcdChangedBlocksDownloadListGetRec
    (HPCDBLOCKDOWNLOADLIST hDnldList, int recNo);
SASM_API BOOL WINAPI PcdChangedBlocksDownloadListGetRecEx(HPCDBLOCKDOWNLOADLIST hDnldList, 
    int recNo, PCDBLOCKDOWNLOADLISTREC* pBlockDownloadListRec);
SASM_API BOOL WINAPI PcdChangedBlocksDownloadListGetUsed
    (HPCDBLOCKDOWNLOADLIST hDnldList, DWORD* pCodeUsed, DWORD* pTextUsed, DWORD* pDataUsed);

// Block Information for actual PCD memory contents

// Block information file record in BlockInfPcd.dat and BlockInfBuild.dat
typedef struct tagPCDBLOCKINFOREC
{
    BYTE segment;       // segment type PCD_CODESEG/PCD_TEXTSEG/PCD_EXTSEG/PCD_IBSEG
    BYTE type;          // block type, PCD_COBTYPE etc
    WORD number;        // block number
    DWORD offset;       // byte offset from start of segment in PCD *memory*
    DWORD size;         // block size in bytes
    DWORD fileOffset;   // byte offset from start of segment in the pcd *file*
    DWORD sum;          // sum of all bytes in block }
    DWORD bcc;          // block check code of block } for 'changed blocks' detection
    WORD crc;           // crc of block              }

} PCDBLOCKINFOREC;

typedef void* HPCDBLOCKINFO;
SASM_API HPCDBLOCKINFO WINAPI PcdBlockInfoOpen(const char* pszCpuPath, int* pErrNo);
SASM_API void WINAPI PcdBlockInfoClose(HPCDBLOCKINFO hInfo);
SASM_API PCDBLOCKINFOREC* WINAPI PcdBlockInfoGet(HPCDBLOCKINFO hInfo, int blockType, int blockNumber);
SASM_API int WINAPI PcdBlockInfoGetEx(HPCDBLOCKINFO hInfo, int blockType, int blockNumber, PCDBLOCKINFOREC* pBlockInfoRec);
SASM_API DWORD WINAPI PcdBlockInfoGetPcdAddsA(HPCDBLOCKINFO hInfo, DWORD dwFileAdds);
SASM_API DWORD WINAPI PcdBlockInfoGetPcdAddsB(HPCDBLOCKINFO hInfo, int blockType, int blockNumber);
SASM_API DWORD WINAPI PcdBlockInfoGetFileAddsA(HPCDBLOCKINFO hInfo, DWORD dwPcdAdds);
SASM_API DWORD WINAPI PcdBlockInfoGetFileAddsB(HPCDBLOCKINFO hInfo, int BlockType, int BlockNumber);
SASM_API LPCTSTR WINAPI PcdGetBlockComment(int blockType, int blockNumber);

//// Fupla 'downloadable in run' flag management
//SASM_API BOOL WINAPI PcdSetFuplaDnldInRun(const char* pszCpuPath, const char* pszModuleName, BOOL bEnabled);
//SASM_API BOOL WINAPI PcdGetFuplaDnldInRun(const char* pszCpuPath, const char* pszModuleName);
//SASM_API BOOL WINAPI PcdGetFuplaDirEditTime(const char* pszCpuPath, const char* pszModuleName, __int64* pTime);


// Downloadable files functions

// Downloadable file record, for PcdFileReadDownloadableFilesList() etc
typedef struct tagPCDDNLDFILEREC
{
    char szRelPath[_MAX_PATH]; // path of file relative to PCD file directory
    DWORD type;                 // file type (file ID)
    DWORD number;               // file number (always 0 for now)
    /*TODO?
    DWORD size;                 // file size in bytes
    DWORD crc;                  // file's crc for change detection
    */
    DWORD status;               // status returned by PcdGetChangedDownloadableFiles()

} PCDDNLDFILEREC;

SASM_API int WINAPI PcdFileReadDownloadableFilesList(const char* pszPcdFilePath, DWORD* pNumDnldFileRecs, PCDDNLDFILEREC** ppDnldFileRecs);
SASM_API BOOL WINAPI PcdFileHasDownloadableFiles(const char* pszPcdFilePath, DWORD* pNumFiles);
SASM_API void WINAPI PcdDownloadableFilesInfoDelete(const char* pszPcdFilePath, DWORD flags);
SASM_API int WINAPI PcdDownloadableFilesBuildInfoCreate(const char* pszPcdFilePath, char* pszErrMsg);
SASM_API int WINAPI PcdDownloadableFilesPcdInfoCreate(const char* pszPcdFilePath);
SASM_API int WINAPI PcdGetChangedDownloadableFiles(const char* pszPcdFilePath, DWORD* pNumDnldFileRecs, PCDDNLDFILEREC** ppDnldFileRecs);
SASM_API int WINAPI PcdFileWriteDownloadableFilesList(const char* pszUplFilePath, DWORD nRecs, PCDDNLDFILEREC** ppRecs);
SASM_API int WINAPI PcdFileGetPcdTypes(const char* pszPcdFilePath, DWORD* pPcdTypes, DWORD* pNumTypes);

// For .NET calls
SASM_API int WINAPI PcdFileReadDownloadableFilesListEx(const char* pszPcdFilePath, DWORD numRecs, PCDDNLDFILEREC* pRecs);
SASM_API int WINAPI PcdGetChangedDownloadableFilesEx(const char* pszPcdFilePath, DWORD numRecs, PCDDNLDFILEREC* pRecs);
SASM_API BOOL WINAPI PcdFileHasChangedDownloadableFiles(const char* pszPcdFilePath, DWORD* pNumFiles);
SASM_API int WINAPI PcdFileWriteDownloadableFilesListEx(const char* pszUplFilePath, DWORD nRecs, PCDDNLDFILEREC* pRecs);

// Flags for PcdDownloadableFilesInfoDelete()
#define PCD_BUILDINFO   0x00000001
#define PCD_PCDINFO     0x00000002

// Cross-reference list functions, new in V2.0
typedef void* HXREFLIST;
SASM_API HXREFLIST WINAPI PcdXrefListOpen(const char* pszPcdFileName, PCDSYM2* pSym, const char* pszModuleName, int* pErrNum);
SASM_API BOOL WINAPI PcdXrefListGetItem(HXREFLIST hList, int item, PCDXREFDATA* pXrefData, char* pszString);
SASM_API void WINAPI PcdXrefListClose(HXREFLIST hList);

// Tag group names
SASM_API int WINAPI PcdDataListHasTagGroupNames(HPCDDATALIST hList, int* pNumTagGroups);
SASM_API int WINAPI PcdDataListGetTagGroupName(HPCDDATALIST hList, int tagIndex, char* pszTagName);
SASM_API int WINAPI PcdDataListGetSymbolsTagGroups(HPCDDATALIST hList, int index, int numTagGroups, WORD* pwTagGroups);
SASM_API int WINAPI PcdDataListGetTagGroupNameEx(HPCDDATALIST hList, int tagIndex, const char** ppszTagName);

// Attributes
SASM_API int WINAPI PcdDataListGetSymbolsAttributes(HPCDDATALIST hList, int index, int numAttributes, const char** ppszAttributes);
SASM_API DWORD WINAPI PcdGetLocalSymbolName(const char* pszInSymbol, char* pszBlockLine, char* pszOutSymbol);

// Expression evaluator
// Does not handle symbols, types or compare operators "= < > <>"
SASM_API int WINAPI EvaluateExpression(char* pszExpression, int* pResult);

// CPU Redundancy
typedef struct tagPCDREDUNDANCYINFO
{
    int Mode;       // 0=not configured, 1=primary device, 2=secondary device, 3=redundant device
    int RecType1;
    DWORD ID1;
    DWORD NumRecs1;
    int RecType2;
    DWORD ID2;
    DWORD NumRecs2;

} PCDREDUNDANCYINFO;

SASM_API int WINAPI PcdFileGetRedundancyInfo(LPCTSTR pszPcdPath, PCDREDUNDANCYINFO* pInfo);

#endif //#if defined(__SCOMM32_H) || defined(SCOMMDLL_API)


// Fupla 'downloadable in run' flag management
SASM_API BOOL WINAPI PcdSetFuplaDnldInRun(const char* pszCpuPath, const char* pszModuleName, BOOL bEnabled);
SASM_API BOOL WINAPI PcdGetFuplaDnldInRun(const char* pszCpuPath, const char* pszModuleName);
SASM_API int WINAPI PcdGetFuplaDnldInRunModCount(const char* pszCpuPath);
//SASM_API BOOL WINAPI PcdGetFuplaDirEditTime(const char* pszCpuPath, const char* pszModuleName, __int64* pi64Time);


///////////////////////////////////////////////////////////////////////////////
// New "static symbol info" functions, V2.3

#define PCDSTATICSYM_FLG_FTI    0x0001  // InitValue is first-time initialization
#define PCDSTATICSYM_FLG_CSI    0x0002  // InitValue is cold-start initialization

typedef struct tagPCDSTATICSYM  // 13 bytes per symbol, in m_pMemBlk
{
    char* pszName;      // symbol name, used as the "ID" (rename disables DIR), string is in m_pMemBlk
    WORD Flags;         // PCDSTATICSYM_FLG_xxx (bits 0..1) SSIFLG_xxx (bits 2..15) see Sload52.dll
    BYTE Type;          // media type, PCDTYPE_xxx
    WORD Address;       // media address 0..65535
    DWORD InitValue;    // first-time or cold-start initialization value, see Flags

} PCDSTATICSYM;

typedef HGLOBAL HPCDSTATICSYMBOLS;  // handle is a pointer to the CPcdStaticSymbols instance

SASM_API HPCDSTATICSYMBOLS WINAPI PcdReadStaticSymbolInfo(const char* pszPCDPath, const char* pszSSIPath, 
                                                          DWORD* pProgramCrc32, int* pNumberOfSymbols);
SASM_API void WINAPI PcdFreeStaticSymbolInfo(HPCDSTATICSYMBOLS h);
SASM_API PCDSTATICSYM* WINAPI PcdFindStaticSymbol(HPCDSTATICSYMBOLS h, const char* pszName);
SASM_API PCDSTATICSYM* WINAPI PcdGetStaticSymbol(HPCDSTATICSYMBOLS h, int index);
SASM_API PCDMODLISTREC* WINAPI PcdGetStaticModListRec(HPCDSTATICSYMBOLS h, int ixModule);

////////////////////////////////////////////////////////////////////////////////
// Functions to read the FTI and CSI information blocks IB 10 and IB 11

// HPCDINITDATA is actually a pointer to the CSymTab instance
typedef HGLOBAL HPCDINITDATA;

SASM_API void WINAPI PcdInitDataFree(HPCDINITDATA h);
SASM_API PCDSTATICSYM* WINAPI PcdInitDataFind(HPCDINITDATA h, int type, int address);
SASM_API PCDSTATICSYM* WINAPI PcdInitDataGetNext(HPCDINITDATA h, BOOL bStart);
SASM_API HPCDINITDATA WINAPI PcdInitDataRead(const char* pszPath, int blockNumber, int* pErrNum);


#if defined(__cplusplus)

// Library file handling
#ifndef UNICODE     // CStringArray and CStringList cannot be used by Unicode apps
class CStringList;
SASM_API BOOL WINAPI PcdCreateLibFilesList(const char* pszProjPath, CStringList& strList);
SASM_API BOOL WINAPI PcdFindLibFile(CStringList& strListLibPaths, void* pIncDirs, 
                                    const char* pszDevicePath, const char* pszCurrentPath, 
                                    const char* pszPathToFind, char* pszPathFound);
SASM_API const char* PcdGetParamTypeString(int paramType);
SASM_API const char* WINAPI PcdGetParamDirectionString(int direction);
#endif

} // ends: extern "C" {
#endif //#if defined(__cplusplus)

#pragma pack(pop)   // end 1-byte packing

#endif // #ifndef __SASM52_H

